package com.apifon.Examples;

import com.apifon.sdk.Mookee;
import com.apifon.sdk.model.IMAction;
import com.apifon.sdk.model.IMChannel;
import com.apifon.sdk.model.ImRequest;
import com.apifon.sdk.model.LandingProperties;
import com.apifon.sdk.model.MessageContent;
import com.apifon.sdk.model.OtpRequest;
import com.apifon.sdk.model.RemoveRequest;
import com.apifon.sdk.model.SmsRequest;
import com.apifon.sdk.model.SubscriberInformation;
import com.apifon.sdk.resource.BalanceResource;
import com.apifon.sdk.resource.IMResource;
import com.apifon.sdk.resource.OTPResource;
import com.apifon.sdk.resource.SMSResource;
import com.apifon.sdk.resource.helpers.DeviceTarget;
import com.apifon.sdk.resource.helpers.RemoveFieldType;
import com.apifon.sdk.response.BalanceResponse;
import com.apifon.sdk.response.GatewayResponse;
import com.apifon.sdk.response.OtpResponse;
import com.apifon.sdk.response.OtpVerifyResponse;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.Locale;
import java.util.Scanner;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * This is a complete example on how to use the APIFON SDK for Java.
 */
public class Example {

    public static void main(String[] args) {
        
        /**
         * First things first, we need to set our credentials that we got from
         * the Mookee platform.
         */
        Mookee.addCredentials("sms", "Your_SMS_Token", "Your_SMS_Secret_Key");
        Mookee.addCredentials("viber", "Your_Viber_Token", "Your_Viber_Secret_Key");
        Mookee.addCredentials("otp", "Your_OTP_Token", "Your_OTP_Secret_Key");

        String choice = "1";
        Scanner in = new Scanner(System.in);

        /**
         * These are our Resources.
         */
        SMSResource smsResource = new SMSResource();
        IMResource imResource = new IMResource();
        OTPResource otpResource = new OTPResource();

        /**
         * These are our Requests.
         */
        SmsRequest smsRequest;
        ImRequest imRequest;
        OtpRequest otpRequest;
        RemoveRequest removeRequest;

        /**
         * Here we generate the date that scheduled messages will be sent.
         */
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.US);
        dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
        String strDate = dateFormat.format(new Date());

        StringBuilder scheduledDate = new StringBuilder(strDate);
        scheduledDate.setCharAt(2, '3');

        /**
         * Here we create our subscribers.
         */
        String[] names = {"John", "George", "Alex", "Helen"};
        String[] nums = {"306999999999"/*, "306999999998", "306999999997", ...*/};
        ArrayList<String> numbers = new ArrayList();
        numbers.addAll(Arrays.asList(nums));

        /**
         * Here we define the content of our Request.
         */
        ArrayList<SubscriberInformation> subscribers;
        IMChannel imChannel;
        IMAction imAction;
        MessageContent msgContent;
        GatewayResponse response;
        OtpResponse otpResp;
        OtpVerifyResponse otpVerResp;
        String delResp;
        String callbackUrl = "https://yourserver/callback";
        String replyUrl = "https://yourserver/replyurl";
        String deletionType;

        while (!choice.equals("0")) {
            System.out.println("------ Menu ------");
            System.out.println("0.  Exit");
            System.out.println("1.  Send Simple SMS");
            System.out.println("2.  Send Complete SMS");
            System.out.println("3.  Send Scheduled SMS");
            System.out.println("4.  Remove Scheduled SMS");
            System.out.println("5.  Send SMS With Landing Page");
            System.out.println("6.  Case 1: Send IM With Text");
            System.out.println("7.  Case 2: Send IM With Image");
            System.out.println("8.  Case 3: Send IM With Text and Button");
            System.out.println("9.  Case 4: Send IM With Text, Image and Button");
            System.out.println("10. Send Complete IM without fallback");
            System.out.println("11. Send Scheduled IM");
            System.out.println("12. Remove Scheduled IM");
            System.out.println("13. Send IM With Landing Page");
            System.out.println("14. Send IM With Fallback SMS");
            System.out.println("15. Send IM With Fallback Landing Page SMS");
            System.out.println("16. Send 2-Way IM");
            System.out.println("17. Send Balance Request");
            System.out.println("18. Send Simple OTP Create Request");
            System.out.println("19. Send Complete OTP Create Request");
            System.out.println("20. Send OTP Verification Request");
            System.out.print("Choice: ");
            choice = in.nextLine();

            /**
             * Every request has the same structure: 
             *  *First we set the Credential we want to use. 
             *  *Then we build the components of the Request. 
             *  *Finally we build the Request and send it. 
             *  *We print the response we get in console.
             */
            switch (choice) {
                //Exit Loop
                case "0":
                    break;
                
                /**
                 * This is a simple SMS Request.
                 */
                case "1":
                    Mookee.setActiveCredential("sms");
                    
                    smsRequest = new SmsRequest();

                    msgContent = new MessageContent();
                    msgContent.setText("Hello There!");
                    msgContent.setSenderId("Apifon");

                    smsRequest.addStrSubscribers(numbers);
                    smsRequest.setMessage(msgContent);
                    smsRequest.setCallbackUrl(callbackUrl);

                    System.out.println("Sending Request to Endpoint: /services/sms/send");
                    System.out.println("------ SMS Request Body ------");
                    printJson(smsRequest);
                    response = smsResource.send(smsRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * In this SMS example we are taking advantage of every property that
                 * can be changed in a Request.
                 */
                case "2":
                    Mookee.setActiveCredential("sms");

                    smsRequest = new SmsRequest();

                    msgContent = new MessageContent();
                    msgContent.setText("Hello {name}!");
                    msgContent.setDc(0);
                    msgContent.setSenderId("Apifon");

                    subscribers = new ArrayList();
                    for (String num : numbers) {
                        subscribers.add(new SubscriberInformation());
                        subscribers.get(subscribers.size() - 1).setNumber(num);
                        subscribers.get(subscribers.size() - 1).setCustomId("222");
                        subscribers.get(subscribers.size() - 1).setPropertyToParam("name", names[subscribers.size() - 1]);
                    }

                    smsRequest.setSubscribers(subscribers);
                    smsRequest.setReferenceId("reference");
                    smsRequest.setMessage(msgContent);
                    smsRequest.setCallbackUrl(callbackUrl);
                    smsRequest.setTte("86400");

                    System.out.println("Sending Request to Endpoint: /services/sms/send");
                    System.out.println("------ SMS Request Body ------");
                    printJson(smsRequest);
                    response = smsResource.send(smsRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * Here we send a scheduled SMS Request.
                 */
                case "3":
                    Mookee.setActiveCredential("sms");

                    smsRequest = new SmsRequest();

                    msgContent = new MessageContent();
                    msgContent.setText("Hello There!");
                    msgContent.setSenderId("Apifon");

                    smsRequest.addStrSubscribers(numbers);
                    smsRequest.setReferenceId("reference");
                    smsRequest.setMessage(msgContent);
                    smsRequest.setCallbackUrl(callbackUrl);
                    smsRequest.setDate(scheduledDate.toString());

                    System.out.println("Sending Request to Endpoint: /services/sms/send");
                    System.out.println("------ SMS Request Body ------");
                    printJson(smsRequest);
                    response = smsResource.send(smsRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * This is an SMS Deletion case.
                 */
                case "4":
                    Mookee.setActiveCredential("sms");
                    
                    deletionType = "";
                    while(!deletionType.equals("0") && !deletionType.equals("1") && !deletionType.equals("2")){
                        System.out.print("Do you want to delete scheduled messages using Request ID or Reference ID?\nEnter [0] for Request ID, [1] for Reference ID or [2] to exit: ");
                        deletionType = in.nextLine();
                        if(!deletionType.equals("0") && !deletionType.equals("1") && !deletionType.equals("2")){
                            System.out.println("Invalid Option.");
                        }
                    }
                    if(!deletionType.equals("2")){
                        removeRequest = new RemoveRequest();
                        if(deletionType.equals("0")){
                            removeRequest.setField(RemoveFieldType.REQUEST_ID.toString());
                            System.out.print("Please enter the Request ID of the scheduled message you want to delete: ");
                            removeRequest.setId(in.nextLine());
                        }
                        else{
                            removeRequest.setField(RemoveFieldType.REFERENCE_ID.toString());
                            System.out.print("Please enter the Reference ID of the scheduled message you want to delete: ");
                            removeRequest.setId(in.nextLine());
                        }
                        System.out.println("Sending Request to Endpoint: /services/sms/remove");
                        System.out.println("------ SMS Remove Request Body ------");
                        printJson(removeRequest);
                        delResp = smsResource.remove(removeRequest);
                        System.out.println("------ SMS Remove Request Response ------\n" + delResp);
                    }
                    break;
                
                /**
                 * In this example we are sending an SMS Request with a landing page.
                 * {apifon_lp} will later be replaced with the URL property of the
                 * landing page.
                 */
                case "5":
                    Mookee.setActiveCredential("sms");

                    smsRequest = new SmsRequest();

                    msgContent = new MessageContent();
                    msgContent.setText("Hello {name}! {apifon_lp}");
                    msgContent.setDc(0);
                    msgContent.setSenderId("Apifon");

                    subscribers = new ArrayList();
                    for (String num : numbers) {
                        subscribers.add(new SubscriberInformation());
                        subscribers.get(subscribers.size() - 1).setNumber(num);
                        subscribers.get(subscribers.size() - 1).setCustomId("222");
                        subscribers.get(subscribers.size() - 1).setPropertyToParam("name", names[subscribers.size() - 1]);

                        LandingProperties lp = new LandingProperties();
                        lp.setUrl("https://mookee.apifon.com/user/login/");
                        lp.addData("name", names[subscribers.size() - 1]);
                        lp.setRedirect(true);

                        subscribers.get(subscribers.size() - 1).setLandingPage(lp);
                    }

                    smsRequest.setSubscribers(subscribers);
                    smsRequest.setReferenceId("reference");
                    smsRequest.setMessage(msgContent);
                    smsRequest.setCallbackUrl(callbackUrl);
                    smsRequest.setTte("86400");

                    System.out.println("Sending Request to Endpoint: /services/sms/send");
                    System.out.println("------ SMS Request Body ------");
                    printJson(smsRequest);
                    response = smsResource.send(smsRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * Viber Case 1: Send IM that contains only a text.
                 */
                case "6":
                    Mookee.setActiveCredential("viber");

                    imRequest = new ImRequest();

                    imChannel = new IMChannel();
                    imChannel.setText("50% discount on our membership fee if you renew your subscription today!");
                    imChannel.setSenderId("Mookee");

                    imRequest.addStrSubscribers(numbers);
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.addImChannel(imChannel);
                    
                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * Viber Case 2: Send IM that contains only an image and define device target to smartphone only.
                 */
                case "7":
                    Mookee.setActiveCredential("viber");

                    imRequest = new ImRequest();

                    imChannel = new IMChannel();
                    imChannel.setSenderId("Mookee");
                    imChannel.addImage("http://assets.apifon.com/images/boy_girl.jpg");
                    imChannel.setDeviceTarget(DeviceTarget.SMARTPHONE_ONLY);

                    imRequest.addStrSubscribers(numbers);
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.addImChannel(imChannel);

                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * Viber Case 3: Send IM that contains text and action button.
                 */
                case "8":
                    Mookee.setActiveCredential("viber");

                    imChannel = new IMChannel();
                    imRequest = new ImRequest();

                    imAction = new IMAction();
                    imAction.setTitle("Renew now!");
                    imAction.setTargetUrl("https://www.apifon.com");

                    imChannel.setSenderId("Mookee");
                    imChannel.setText("50% discount on our membership fee if you renew your subscription today!");
                    imChannel.addAction(imAction);

                    imRequest.addStrSubscribers(numbers);
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.addImChannel(imChannel);

                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * Viber Case 4: Send IM that contains text, image and action button.
                 */
                case "9":
                    Mookee.setActiveCredential("viber");

                    imChannel = new IMChannel();
                    imRequest = new ImRequest();

                    imAction = new IMAction();
                    imAction.setTitle("Renew now!");
                    imAction.setTargetUrl("https://www.apifon.com");

                    imChannel.setSenderId("Mookee");
                    imChannel.setText("50% discount on our membership fee if you renew your subscription today!");
                    imChannel.addImage("http://assets.apifon.com/images/boy_girl.jpg");
                    imChannel.addAction(imAction);

                    imRequest.addStrSubscribers(numbers);
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.addImChannel(imChannel);

                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * Complete Viber example.
                 */
                case "10":
                    Mookee.setActiveCredential("viber");

                    imChannel = new IMChannel();
                    imRequest = new ImRequest();

                    imAction = new IMAction();
                    imAction.setTitle("Renew now!");
                    imAction.setTargetUrl("https://www.apifon.com");

                    imChannel.setId("channelID");
                    imChannel.setSenderId("Mookee");
                    imChannel.setText("{name}! 50% discount on our membership fee if you renew your subscription today!");
                    imChannel.addImage("http://assets.apifon.com/images/boy_girl.jpg");
                    imChannel.addAction(imAction);
                    imChannel.setTtl(86400);
                    imChannel.setExpiryText("Message Expired!!");

                    subscribers = new ArrayList();
                    for (String num : numbers) {
                        subscribers.add(new SubscriberInformation());
                        subscribers.get(subscribers.size() - 1).setNumber(num);
                        subscribers.get(subscribers.size() - 1).setCustomId("222");
                        subscribers.get(subscribers.size() - 1).setPropertyToParam("name", names[subscribers.size() - 1]);
                    }

                    imRequest.setSubscribers(subscribers);
                    imRequest.setReferenceId("reference");
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.addImChannel(imChannel);

                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * Here we send a scheduled IM Request.
                 */
                case "11":
                    Mookee.setActiveCredential("viber");

                    imChannel = new IMChannel();
                    imRequest = new ImRequest();

                    imAction = new IMAction();
                    imAction.setTitle("Renew now!");
                    imAction.setTargetUrl("https://www.apifon.com");

                    imChannel.setSenderId("Mookee");
                    imChannel.setText("50% discount on our membership fee if you renew your subscription today!");
                    imChannel.addImage("http://assets.apifon.com/images/boy_girl.jpg");
                    imChannel.addAction(imAction);

                    imRequest.addStrSubscribers(numbers);
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.addImChannel(imChannel);
                    imRequest.setDate(scheduledDate.toString());

                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * This is an IM Deletion case.
                 */
                case "12":
                    Mookee.setActiveCredential("viber");

                    deletionType = "";
                    while(!deletionType.equals("0") && !deletionType.equals("1") && !deletionType.equals("2")){
                        System.out.print("Do you want to delete scheduled messages using Request ID or Reference ID?\nEnter [0] for Request ID, [1] for Reference ID or [2] to exit: ");
                        deletionType = in.nextLine();
                        if(!deletionType.equals("0") && !deletionType.equals("1") && !deletionType.equals("2")){
                            System.out.println("Invalid Option.");
                        }
                    }
                    if(!deletionType.equals("2")){
                        removeRequest = new RemoveRequest();
                        if(deletionType.equals("0")){
                            removeRequest.setField(RemoveFieldType.REQUEST_ID.toString());
                            System.out.print("Please enter the Request ID of the scheduled message you want to delete: ");
                            removeRequest.setId(in.nextLine());
                        }
                        else{
                            removeRequest.setField(RemoveFieldType.REFERENCE_ID.toString());
                            System.out.print("Please enter the Reference ID of the scheduled message you want to delete: ");
                            removeRequest.setId(in.nextLine());
                        }
                        System.out.println("Sending Request to Endpoint: /services/im/remove");
                        System.out.println("------ IM Remove Request Body ------");
                        printJson(removeRequest);
                        delResp = imResource.remove(removeRequest);
                        System.out.println("------ IM Remove Request Response ------\n" + delResp);
                    }
                    break;
                
                /**
                 * In this example we send a Viber message containing a landing page.
                 */
                case "13":
                    Mookee.setActiveCredential("viber");

                    imChannel = new IMChannel();
                    imRequest = new ImRequest();

                    imAction = new IMAction();
                    imAction.setTitle("Renew now!");
                    imAction.setTargetUrl("{apifon_lp}");

                    imChannel.setSenderId("Mookee");
                    imChannel.setText("{name}! 50% discount on our membership fee if you renew your subscription today!");
                    imChannel.addImage("http://assets.apifon.com/images/boy_girl.jpg");
                    imChannel.addAction(imAction);

                    subscribers = new ArrayList();
                    for (String num : numbers) {
                        subscribers.add(new SubscriberInformation());
                        subscribers.get(subscribers.size() - 1).setNumber(num);
                        subscribers.get(subscribers.size() - 1).setCustomId("222");
                        subscribers.get(subscribers.size() - 1).setPropertyToParam("name", names[subscribers.size() - 1]);

                        LandingProperties lp = new LandingProperties();
                        lp.setUrl("https://mookee.apifon.com/user/login/");
                        lp.addData("name", names[subscribers.size() - 1]);
                        lp.setRedirect(true);

                        subscribers.get(subscribers.size() - 1).setLandingPage(lp);
                    }

                    imRequest.setSubscribers(subscribers);
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.addImChannel(imChannel);

                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * In this example we send a Viber message with a fallback SMS, in
                 * case Viber delivery fails.
                 */
                case "14":
                    Mookee.setActiveCredential("viber");

                    imChannel = new IMChannel();
                    imRequest = new ImRequest();

                    imAction = new IMAction();
                    imAction.setTargetUrl("https://www.apifon.com");
                    imAction.setTitle("Renew now!");

                    imChannel.setSenderId("Mookee");
                    imChannel.setText("50% discount on our membership fee if you renew your subscription today!");
                    imChannel.addImage("http://assets.apifon.com/images/boy_girl.jpg");
                    imChannel.addAction(imAction);

                    msgContent = new MessageContent();
                    msgContent.setText("Hello There!");
                    msgContent.setSenderId("Apifon");

                    imRequest.addStrSubscribers(numbers);
                    imRequest.setMessage(msgContent);
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.addImChannel(imChannel);

                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * In this example we send a Viber message with a landing page
                 * fallback SMS, in case Viber delivery fails.
                 */
                case "15":
                    Mookee.setActiveCredential("viber");

                    imChannel = new IMChannel();
                    imRequest = new ImRequest();

                    imAction = new IMAction();
                    imAction.setTitle("Renew now!");
                    imAction.setTargetUrl("https://www.apifon.com");

                    imChannel.setSenderId("Mookee");
                    imChannel.setText("50% discount on our membership fee if you renew your subscription today!");
                    imChannel.addImage("http://assets.apifon.com/images/boy_girl.jpg");
                    imChannel.addAction(imAction);

                    subscribers = new ArrayList();
                    for (String num : numbers) {
                        subscribers.add(new SubscriberInformation());
                        subscribers.get(subscribers.size() - 1).setNumber(num);
                        subscribers.get(subscribers.size() - 1).setCustomId("222");
                        subscribers.get(subscribers.size() - 1).setPropertyToParam("name", names[subscribers.size() - 1]);

                        LandingProperties lp = new LandingProperties();
                        lp.setUrl("https://mookee.apifon.com/user/login/");
                        lp.addData("name", names[subscribers.size() - 1]);
                        lp.setRedirect(true);

                        subscribers.get(subscribers.size() - 1).setLandingPage(lp);
                    }

                    msgContent = new MessageContent();
                    msgContent.setText("Hello {name}! {apifon_lp}");
                    msgContent.setSenderId("Apifon");

                    imRequest.setSubscribers(subscribers);
                    imRequest.setMessage(msgContent);
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.addImChannel(imChannel);

                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * This is a 2-way Viber example. Any replies sent by the 
                 * message recipient will generate a reply-message callback 
                 * on the replyURL. Note that this request would require use of 
                 * a Viber Sender ID with 2-way messaging enabled. 2-way 
                 * messaging can be enabled on a Sender ID via our platform.
                 */
                case "16":
                    Mookee.setActiveCredential("viber");

                    imChannel = new IMChannel();
                    imRequest = new ImRequest();

                    imAction = new IMAction();
                    imAction.setTitle("Renew now!");
                    imAction.setTargetUrl("https://www.apifon.com");

                    imChannel.setSenderId("Mookee");
                    imChannel.setText("50% discount on our membership fee if you renew your subscription today! Would you like to renew your subscription?");
                    imChannel.addImage("http://assets.apifon.com/images/boy_girl.jpg");
                    imChannel.addAction(imAction);

                    imRequest.addStrSubscribers(numbers);
                    imRequest.setCallbackUrl(callbackUrl);
                    imRequest.setReplyUrl(replyUrl);
                    imRequest.addImChannel(imChannel);

                    System.out.println("Sending Request to Endpoint: /services/im/send");
                    System.out.println("------ IM Request Body ------");
                    printJson(imRequest);
                    response = imResource.send(imRequest);
                    System.out.println("------ Gateway Response Body ------");
                    printJson(response);
                    break;
                
                /**
                 * This is a Balance Request example in which we are informed about our
                 * remaining Balance and our Plafon.
                 */
                case "17":
                    Mookee.setActiveCredential("sms");

                    BalanceResource balanceReq = new BalanceResource();
                    System.out.println("Sending Request to Endpoint: /services/balance");
                    System.out.println("------ Balance Request Body ------\nEmpty Body");
                    BalanceResponse balanceResp = balanceReq.send();
                    System.out.println("------ Balance Response Body ------");
                    printJson(balanceResp);
                    break;
                    
                /**
                 * This is a basic OTP Create Request example. We are only setting a phone
                 * number and a reference_id. Reference_id is important because it links to 
                 * the generated OTP.
                 */
                case "18":
                    Mookee.setActiveCredential("otp");
                    
                    otpRequest = new OtpRequest();
                    
                    System.out.print("Enter a reference_id for your OTP create request: ");
                    otpRequest.setReferenceId(in.nextLine());
                    otpRequest.setSubscriber(nums[0]);
                    
                    System.out.println("Sending Request to Endpoint: /services/otp/create");
                    System.out.println("------ OTP Create Request Body ------");
                    printJson(otpRequest);
                    otpResp = otpResource.create(otpRequest);
                    System.out.println("------ OTP Create Response Body ------");
                    printJson(otpResp);
                    break;
                    
                /**
                 * This is a complete OTP Create Request example. We are specifying every
                 * field of the request to our preferences.
                 */
                case "19":
                    Mookee.setActiveCredential("otp");
                    
                    otpRequest = new OtpRequest();
                    
                    msgContent = new MessageContent();
                    msgContent.setText("Hello There! Here is your OTP: {apifon_otp}");
                    msgContent.setSenderId("Apifon");
                    
                    System.out.print("Enter a reference_id for your OTP create request: ");
                    otpRequest.setReferenceId(in.nextLine());
                    otpRequest.setSubscriber(nums[0]);
                    otpRequest.setCodeLenght(6);
                    otpRequest.setCodeType("numeric");
                    otpRequest.setCallbackUrl(callbackUrl);
                    otpRequest.setExpire(300);
                    otpRequest.setMessage(msgContent);
                    
                    System.out.println("Sending Request to Endpoint: /services/otp/create");
                    System.out.println("------ OTP Create Request Body ------");
                    printJson(otpRequest);
                    otpResp = otpResource.create(otpRequest);
                    System.out.println("------ OTP Create Response Body ------");
                    printJson(otpResp);
                    break;
                    
                /**
                 * This is an OTP Verification Request example. When a subscriber is using
                 * the OTP that we generated for him, we want to verify that the OTP is valid.
                 * We are using the reference_id that we used to generate the OTP and the OTP that
                 * we are provided to make this verification.
                 */
                case "20":
                    Mookee.setActiveCredential("otp");
                    
                    System.out.print("Enter the reference_id of the OTP you want to verify: ");
                    String otpReference = in.nextLine();
                    System.out.print("Enter the OTP you want to verify: ");
                    String otpCode = in.nextLine();
                    
                    System.out.println("Sending Request to Endpoint: /services/otp/verify/" + otpReference + "/" + otpCode);
                    System.out.println("------ OTP Verify Request Body ------\nEmpty Body");
                    otpVerResp = otpResource.verify(otpReference, otpCode);
                    System.out.println("------ OTP Verify Response Body ------");
                    printJson(otpVerResp);
                    break;
                default:
                    System.out.println("Not a valid option\n");
                    break;
            }
            try {
                TimeUnit.SECONDS.sleep(1);
            } catch (InterruptedException ex) {
                Logger.getLogger(Example.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    //This Method is used to Print our Json objects, Requests and Responses.
    private static void printJson(Object object) {
        Gson gson = new GsonBuilder().setPrettyPrinting().create();
        JsonParser jp = new JsonParser();
        JsonElement je = jp.parse(gson.toJson(object));
        String prettyJsonString = gson.toJson(je);
        System.out.println(prettyJsonString);
    }
}
