/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.Examples;

import com.apifon.sdk.Mookee;
import com.apifon.sdk.exception.ApifonRestException;
import com.apifon.sdk.model.request.*;
import com.apifon.sdk.resource.*;
import com.apifon.sdk.resource.helpers.*;
import com.apifon.sdk.response.*;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.ThreadLocalRandom;

/**
 *
 * @author Apifon
 */
public class ListApiExample {

    // Resources
    // We use those to dispatch requests to the API.
    private static final ListResource LIST_RESOURCE = new ListResource();
    private static final SubscriberFieldResource SUBSCRIBER_FIELD_RESOURCE = new SubscriberFieldResource();
    private static final GdprFieldResource GDPR_FIELD_RESOURCE = new GdprFieldResource();
    private static final SubscriberResource SUBSCRIBER_RESOURCE = new SubscriberResource();
    private static final GroupResource GROUP_RESOURCE = new GroupResource();
    private static final SubgroupResource SUBGROUP_RESOURCE = new SubgroupResource();

    private static final Gson GSON = new GsonBuilder().setPrettyPrinting().create();

    public static void main(String[] args) throws URISyntaxException {
        try {
            Mookee.addCredentials("token", "MY_TOKEN", "MY_SECRET_KEY");
            Mookee.setActiveCredential("token");
            // Uncomment this line to enable staging requests.
            // Mookee.setStaging(true, null);
            // Create a List
            ListResponse createdList = createList();
            System.out.println("Created List: " + GSON.toJson(createdList));
            // Add a GDPR and a Subscriber field to our list.
            addSubscriberFieldAndGdprField(createdList.getId());
            // Add a group and subgroups to the our list
            addGroupAndSubgroups(createdList.getId());
            // Add a subscriber
            SubscriberResponse createdSubscriber = addSubscriber(createdList.getId());
            System.out.println("Added Subscriber: " + GSON.toJson(createdSubscriber));
            // Subscribe his email address.
            subscriberSubscriber(createdList.getId(), createdSubscriber.getEmailId());
        } catch (ApifonRestException ex) {
            System.out.println(ex.toString());
        }
    }

    /**
     * Used to create a list
     * @return the List that was created
     * @throws ApifonRestException if an exception occured
     */
    private static ListResponse createList() throws ApifonRestException {
        ListRequestBuilder.ListRequest request = ListRequestBuilder.getAddListRequestBuilder(ListType.ADVANCED, "List created from SDK", "30")
                .withDescription("This is a list created using the SDK.")
                .withCampaignDefaults(
                        CampaignDefaultsBuilder.getCampaignDefaultsBuilder()
                                .setFrom("email@apifon.com")
                                .setReplyTo("noreply@apifon.com")
                                .setSenderName("Apifon")
                                .setSenderId("Apifon")
                                .build()
                )
                .withGdprProperties(
                        GdprPropertiesBuilder.getGdprPropertiesBuilder(true)
                                .setLegalText("My Legal Text")
                                .setTitle("Communication Purposes")
                                .setDescription("Information about the purposes we wish to contact you.")
                )
                .build();
        URI createdListResource = LIST_RESOURCE.createList(request);
        return LIST_RESOURCE.getByResourceUri(createdListResource);
    }

    /**
     * Adds a field and a GDPR field to the list with the given ID.
     * @param listId the list that will be updated
     * @throws ApifonRestException if an exception occured
     */
    private static void addSubscriberFieldAndGdprField(String listId) throws ApifonRestException {
        // Generate a SubscriberFieldRequest using SubscriberFieldRequestBuilder
        SubscriberFieldRequestBuilder.SubscriberFieldRequest subFieldRequest
                = SubscriberFieldRequestBuilder.getAddRequestBuilder(listId, SubscriberFieldType.TEXT, 20, "First Name")
                        .withInternal(false)
                        .build();
        SUBSCRIBER_FIELD_RESOURCE.addSubscriberField(subFieldRequest);

        // Generate a GdprFieldRequest using GdprFieldRequestBuilder
        GdprFieldRequestBuilder.GdprFieldRequest gdprFieldRequest
                = GdprFieldRequestBuilder.getAddGdprFieldRequestBuilder("Offers", listId)
                        .withDescription("Information about our running offers.")
                        .build();
        GDPR_FIELD_RESOURCE.addGdprField(gdprFieldRequest);
    }
    /**
     * Adds a group  to the list and two subgroups to that group with the given ID.
     * @param listId the list that will be updated
     * @throws ApifonRestException if an exception occured
     */
    private static void addGroupAndSubgroups(String listId) throws ApifonRestException {
        //generate a group using the group request builder
        GroupRequestBuilder groupRequestBuilder = GroupRequestBuilder.getAddGroupRequestBuilder(listId, GroupType.GROUP,"my group")
                .withSelectionType(SelectionType.MULTIPLE);
        URI groupURI = GROUP_RESOURCE.addGroup(groupRequestBuilder.build());

        //request the new group back to get the group field needed to add the subgroup options
        String groupField = GROUP_RESOURCE.getByResourceUri(groupURI).getField();

        //generate two groups using the subgroup request builder
        List<String> mySubgroups = new ArrayList<>();
        mySubgroups.add("option 1");
        mySubgroups.add("option 2");
        SubgroupRequestBuilder subgroupRequestBuilder = SubgroupRequestBuilder.getAddSubgroupsRequestBuilder(listId,groupField,mySubgroups);
        SUBGROUP_RESOURCE.addSubgroups(subgroupRequestBuilder.build());

    }

    /**
     * Adds a Subscriber to the list with the given list ID.
     * @param listId the list ID that the subscriber will be added to
     * @return The subscriber that was added
     * @throws ApifonRestException if an exception occured
     */
    private static SubscriberResponse addSubscriber(String listId) throws ApifonRestException {
        // First let's retrieve the list!
        ListSingleQuery listRequest = ListSingleQuery.getListRequest(listId);
        ListResponse list = LIST_RESOURCE.getList(listRequest);
        // Create a SubscriberRequestBuilder
        SubscriberRequestBuilder requestBuilder = SubscriberRequestBuilder.getAddSubscriberRequestBuilder(listId)
                .withDestination("30699999999", SubscriberStatus.SUBSCRIBED)
                .withEmail("mail@apifon.com", SubscriberStatus.UNSUBSCRIBED);
        // Setting gdpr compliance for all list gdpr fields.
        if (list.getGdprFields() != null) {
            for (GdprFieldResponse resp : list.getGdprFields()) {
                requestBuilder.withGdprField(resp.getField(), true);
            }
        }
        // Setting random values for every subscriber field.
        if (list.getSubscriberFields() != null) {
            for (SubscriberFieldResponse resp : list.getSubscriberFields()) {
                requestBuilder.withSubscriberField(resp.getField(), getRandomSubscriberValue(resp.getType()));
            }
        }

        List<String> groupOptions = new ArrayList<>();
        if(list.getGroups() != null){
            for (GroupResponse group : list.getGroups()){
                if(group.getOptions() != null){
                    for(SubgroupResponse option : group.getOptions()){
                        groupOptions.add(option.getField());
                    }
                }
            }
        }
        requestBuilder.withGroupOptions(groupOptions);

        SubscriberRequestBuilder.SubscriberRequest request = requestBuilder.build();
        URI addSubscriberResource = SUBSCRIBER_RESOURCE.addSubscriber(request);
        return SUBSCRIBER_RESOURCE.getByResourceUri(addSubscriberResource);
    }
    
    /**
     * Subscribers the subscriber with the given lookupMD5 to the list given.
     * @param listId the list ID of the subscriber
     * @param lookupMD5 the unique MD5 hash of the subscriber. Valid values are {@link SubscriberResponse#destinationId} or {@link SubscriberResponse#emailId}
     * @throws ApifonRestException if an exception occured
     */
    private static void subscriberSubscriber(String listId, String lookupMD5) throws ApifonRestException {
        SUBSCRIBER_RESOURCE.subscribeSubscriber(listId, lookupMD5);
    }

    private static Object getRandomSubscriberValue(SubscriberFieldType type) {
        String randUUID = UUID.randomUUID().toString();
        switch (type) {
            case NUMBER:
                return ThreadLocalRandom.current().nextInt(100);
            case TEXT:
                return randUUID;
            case DATE:
                return new Date();
            case EMAIL:
                return randUUID + "@domain.com";
            case URL:
                return "http://192.168.1.1/" + randUUID;
            case MOBILE_NUMBER:
                return ThreadLocalRandom.current().nextInt(100);
            default:
                return randUUID;
        }
    }
}
