/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.Examples;

import com.apifon.sdk.Mookee;
import com.apifon.sdk.exception.ApifonRestException;
import com.apifon.sdk.exception.OAuthException;
import com.apifon.sdk.model.request.CampaignDefaultsBuilder;
import com.apifon.sdk.model.request.GdprPropertiesBuilder;
import com.apifon.sdk.model.request.ListRequestBuilder;
import com.apifon.sdk.model.request.OAuthConfigurationBuilder;
import com.apifon.sdk.model.request.OAuthConfigurationBuilder.OAuthConfiguration;
import com.apifon.sdk.resource.ListResource;
import com.apifon.sdk.resource.helpers.ListType;
import com.apifon.sdk.response.ListResponse;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Apifon
 */
public class OauthExample {

    private static final String OAUTH_IDENTIFIER = "YOUR_IDENTIFIER";
    private static final String CLIENT_ID = "YOUR_CLIENT_ID";
    private static final String CLIENT_SECRET = "YOUR_CLIENT_SECRET";
    // Using a ListResource to call list-related APIs
    private static final ListResource LIST_RESOURCE = new ListResource();

    public static void main(String[] args) throws Exception {
        clientCredentialsGrantExample();
    }

    private static void clientCredentialsGrantExample() throws OAuthException, URISyntaxException, IOException {
        // Create an OAuth configuration to use
        OAuthConfiguration oauthConfig = getClientCredentialsGrantConfiguration(
                CLIENT_ID, 
                CLIENT_SECRET
        );
        // Setting the OAuth configuration to Mookee instance!
        Mookee.addOAuthResource(OAUTH_IDENTIFIER, oauthConfig);
        // Retrieve an OAuth2 token from the Identity server using the configuration.
        Mookee.retrieveClientCredentialsToken(OAUTH_IDENTIFIER);
        // Setting the Mookee to OAuth mode so that OAuth is used for any subsequent API calls.
        Mookee.setMookeeToOAuthMode(OAUTH_IDENTIFIER, true);
        // Call API to create List
        ListResponse createListResponse = createList();
        System.out.println("Created list with ID " + createListResponse.getId() + " and title " + createListResponse.getTitle());
        // Revoke active tokens!
        Mookee.removeOAuthResource(OAUTH_IDENTIFIER, true);
    }

    private static ListResponse createList() throws ApifonRestException {
        ListRequestBuilder.ListRequest request = ListRequestBuilder.getAddListRequestBuilder(ListType.ADVANCED, "OAuth2 List", "30")
                .withDescription("This is a list created using the SDK and an OAuth2 Bearer token!")
                .withCampaignDefaults(
                        CampaignDefaultsBuilder.getCampaignDefaultsBuilder()
                                .setFrom("email@apifon.com")
                                .setReplyTo("noreply@apifon.com")
                                .setSenderName("Apifon")
                                .setSenderId("Apifon")
                                .build()
                )
                .withGdprProperties(
                        GdprPropertiesBuilder.getGdprPropertiesBuilder(true)
                                .setLegalText("My Legal Text")
                                .setTitle("Communication Purposes")
                                .setDescription("Information about the purposes we wish to contact you.")
                )
                .build();
        URI createdListResource = LIST_RESOURCE.createList(request);
        return LIST_RESOURCE.getByResourceUri(createdListResource);
    }

    private static OAuthConfiguration getClientCredentialsGrantConfiguration(String clientId, String clientSecret) {
        return OAuthConfigurationBuilder
                .getClientCredentialsBuilder(clientId, clientSecret)
                .build();
    }
}
