/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package com.apifon.sdk.exception;

import com.apifon.sdk.response.ApiErrorResponse;
import java.io.IOException;
import java.util.Map;

/**
 * An exception wrapping up validation errors thrown by Apifon REST API and IOExceptions
 * caused while deserializing responses.<br><br>
 * Contains an entity of ApiErrorResponse containing details about the error OR the original IOException
 * that caused this exception to be thrown.
 * @author Apifon
 */
public class ApifonRestException extends IOException {
    
    private final ApiErrorResponse resp;
    
    public ApifonRestException(String message) {
        super(message);
        resp = null;
    }
    
    public ApifonRestException(String message, IOException ex) {
        super(message, ex);
        resp = null;
    }
    
    public ApifonRestException(String message, IOException ex, ApiErrorResponse resp) {
        super(message, ex);
        this.resp = resp;
    }
    
    public ApifonRestException(IOException ex) {
        super(ex);
        resp = null;
    }
    
    public ApifonRestException(IOException ex, ApiErrorResponse resp) {
        super(ex);
        this.resp = resp;
    }

    public ApifonRestException(ApiErrorResponse resp) {
        this.resp = resp;
    }
    
    public ApifonRestException(String message, ApiErrorResponse resp) {
        super(message);
        this.resp = resp;
    }

    public ApiErrorResponse getResp() {
        return resp;
    }
    
    /**
     * Prints formatted readable information about the error.
     * @return 
     */
    @Override
    public String toString() {
        if (resp != null) {
            StringBuilder errorDetails = new StringBuilder();
            for (Map.Entry<String, String> pair : resp.errorDetails.entrySet()) {
                errorDetails.append(pair.getKey()).append(":").append(pair.getValue()).append(System.lineSeparator());
            }
            return   "Status: " + String.valueOf(resp.status) + System.lineSeparator()
                    + "Title: " + resp.title + System.lineSeparator()
                    + "Error details: " + errorDetails.toString()
                    + "Error code: " + resp.errorReportCode;
        } else {
            return super.getMessage();
        }
    }

}
