package com.apifon.sdk.model;

import com.apifon.sdk.resource.helpers.DeviceTarget;
import com.google.gson.annotations.SerializedName;
import java.util.ArrayList;
import java.util.List;

/**
 * IMChannel class represents an im_channel Property in an IM Request.
 */
public class IMChannel {

    private String id;
    @SerializedName("sender_id")
    private String senderId;
    private String text;
    private List<String> images;
    private List<IMAction> actions;
    private Integer ttl;
    @SerializedName("expiry_text")
    private String expiryText;
    @SerializedName("device_target")
    private DeviceTarget deviceTarget;

    public IMChannel() {
    }

    /**
     * Returns the id Parameter.
     *
     * @return the id.
     */
    public String getId() {
        return id;
    }

    /**
     * Sets the id Parameter.
     *
     * @param id the id to set.
     */
    public void setId(String id) {
        this.id = id;
    }

    /**
     * Returns the senderId Parameter.
     *
     * @return the senderId.
     */
    public String getSenderId() {
        return senderId;
    }

    /**
     * Sets the senderId Parameter.
     *
     * @param senderId the senderId to set.
     */
    public void setSenderId(String senderId) {
        this.senderId = senderId;
    }

    /**
     * Returns the text Parameter.
     *
     * @return the text.
     */
    public String getText() {
        return text;
    }

    /**
     * Sets the text Parameter.
     *
     * @param text the text to set.
     */
    public void setText(String text) {
        this.text = text;
    }

    /**
     * Returns the images Parameter.
     *
     * @return the images.
     */
    public List<String> getImages() {
        return images;
    }

    /**
     * Sets the images Parameter.
     *
     * @param images the images to set.
     */
    public void setImages(List<String> images) {
        this.images = images;
    }

    /**
     * Adds an image to the images list.
     *
     * @param image the images to add.
     */
    public void addImage(String image) {
        if(this.images == null){
            this.images = new ArrayList();
        }
        this.images.add(image);
    }

    /**
     * Returns the actions[] Parameter.
     *
     * @return the actions.
     */
    public List<IMAction> getActions() {
        return actions;
    }

    /**
     * Sets the actions[] Parameter.
     *
     * @param actions the actions to set.
     */
    public void setActions(List<IMAction> actions) {
        this.actions = actions;
    }

    /**
     * Adds an IMAction to the actions list.
     *
     * @param action the actions to add.
     */
    public void addAction(IMAction action) {
        if(this.actions == null){
            this.actions = new ArrayList();
        }
        this.actions.add(action);
    }

    /**
     * Returns the TTL Parameter.
     *
     * @return the TTL.
     */
    public int getTtl() {
        return ttl;
    }

    /**
     * Sets the TTL Parameter.
     *
     * @param ttl the TTL to set.
     */
    public void setTtl(int ttl) {
        this.ttl = ttl;
    }

    /**
     * Returns the expiryText Parameter.
     *
     * @return the expiryText.
     */
    public String getExpiryText() {
        return expiryText;
    }

    /**
     * Sets the expiryText Parameter.
     *
     * @param expiryText the expiryText to set.
     */
    public void setExpiryText(String expiryText) {
        this.expiryText = expiryText;
    }

    /**
     * Returns the deviceTarget Parameter.
     *
     * @return the deviceTarget.
     */
    public DeviceTarget getDeviceTarget(){
        return deviceTarget;
    }
    
     /**
     * Sets the deviceTarget Parameter.
     *
     * @param deviceTarget the deviceTarget to set.
     */
    public void setDeviceTarget(DeviceTarget deviceTarget){
        this.deviceTarget = deviceTarget;
    }
}
