package com.apifon.sdk.model;

import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import java.util.ArrayList;
import java.util.List;

/**
 * ImRequest class represents an IM Request to the APIFON Rest API. You can
 * create an IMResource object and call it's send() method, using an ImRequest
 * object as a parameter.
 */
public class ImRequest extends SubscribersViewRequest {

    @SerializedName("reference_id")
    private String referenceId;
    private MessageContent message;
    @SerializedName("callback_url")
    private String callbackUrl;
    @SerializedName("reply_url")
    private String replyUrl;
    @SerializedName("im_channels")
    private List<IMChannel> imChannels;
    private String date;

    public ImRequest() {
        this.message = new MessageContent();
        this.imChannels = new ArrayList<>();
    }

    /**
     * Returns the message Parameter.
     *
     * @return the message.
     */
    public MessageContent getMessage() {
        return message;
    }

    /**
     * Sets the message Parameter.
     *
     * @param message the message to set.
     */
    public void setMessage(MessageContent message) {
        this.message = message;
    }

    /**
     * Returns the callbackUrl Parameter.
     *
     * @return the callbackUrl.
     */
    public String getCallbackUrl() {
        return callbackUrl;
    }

    /**
     * Sets the callbackUrl Parameter.
     *
     * @param callbackUrl the callbackUrl to set.
     */
    public void setCallbackUrl(String callbackUrl) {
        this.callbackUrl = callbackUrl;
    }

    /**
     * Returns the replyUrl Parameter.
     *
     * @return the replyUrl.
     */
    public String getReplyUrl() {
        return replyUrl;
    }

    /**
     * Sets the replyUrl Parameter.
     *
     * @param replyUrl the replyUrl to set.
     */
    public void setReplyUrl(String replyUrl) {
        this.replyUrl = replyUrl;
    }
    
    /**
     * Returns the referenceId Parameter.
     *
     * @return the referenceId.
     */
    public String getReferenceId() {
        return referenceId;
    }

    /**
     * Sets the referenceId Parameter.
     *
     * @param referenceId the referenceId to set.
     */
    public void setReferenceId(String referenceId) {
        this.referenceId = referenceId;
    }

    /**
     * Returns the imChannels Parameter.
     *
     * @return the imChannels.
     */
    public List<IMChannel> getImChannels() {
        return imChannels;
    }

    /**
     * Returns the imChannels Parameter.
     *
     * @param imChannels the imChannels to set.
     */
    public void setImChannels(List<IMChannel> imChannels) {
        this.imChannels = imChannels;
    }

    /**
     * Adds an imChannel to the list.
     *
     * @param im_channel the im_channel to add to the list.
     */
    //validateIMChannels
    public void addImChannel(IMChannel im_channel) {
        imChannels.add(im_channel);
    }

    /**
     * Adds an imChannel to the list.
     *
     * @param im_channel the im_channel in a JSon form to add to the list.
     */
    public void setStrImChannel(String im_channel) {
        Gson gson = new Gson();
        IMChannel tmp = gson.fromJson(im_channel, IMChannel.class);
        imChannels.add(tmp);
    }

    /**
     * Returns the Date Parameter.
     *
     * @return the Date.
     */
    public String getDate() {
        return date;
    }

    /**
     * Sets the Date Parameter.
     *
     * @param date the Date to set.
     */
    public void setDate(String date) {
        this.date = date;
    }

    @Override
    public String getBody() {
        Gson gson = new Gson();
        return gson.toJson(this);
    }

    @Override
    public String getCreateBody() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public String getReadBody() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public String getUpdateBody() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public String getDeleteBody() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

}
