/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.model.request;

import com.apifon.sdk.resource.helpers.PaginatedRequest;
import com.apifon.sdk.resource.helpers.PaginatedRequestBuilder;
import com.apifon.sdk.resource.helpers.BounceReason;
import com.apifon.sdk.resource.helpers.BounceType;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.message.BasicNameValuePair;
import com.apifon.sdk.resource.helpers.IFieldSerializable;
import java.util.Collection;

/**
 * A builder used to generate bounce queries for use with the API.
 * @author Apifon
 */
public class BounceQueryBuilder extends PaginatedRequestBuilder implements IFieldSerializable<BounceQueryBuilder> {

    private BounceType type;
    private BounceReason reason;
    private final Set<String> fields;

    private BounceQueryBuilder() {
        this.fields = new HashSet<>();
    }

    public static BounceQueryBuilder getBuilder() {
        return new BounceQueryBuilder();
    }

    /**
     * Sets query with a specific type [EMAIL, MOBILE]
     * @param type
     * @return this for chaining
     * @throws NullPointerException if null type is provided.
     */
    public BounceQueryBuilder withType(BounceType type) throws NullPointerException {
        Objects.requireNonNull(type, "Parameter type is null.");
        this.type = type;
        return this;
    }

    /**
     * Sets query with a specific reason
     * @see BounceReason
     * @param reason
     * @return
     * @throws NullPointerException if null reason is provided.
     */
    public BounceQueryBuilder withReason(BounceReason reason) throws NullPointerException {
        Objects.requireNonNull(reason, "Parameter reason is null.");
        this.reason = reason;
        return this;
    }

    /**
     * Builds and returns the request.
     * @return 
     */
    @Override
    public QueryBounceSubscribersRequest build() {
        return new QueryBounceSubscribersRequest(this);
    }

    /**
     * Buildes the request and returns the query parameters.
     * @return 
     */
    @Override
    public String toQueryParamRequest() {
        return new QueryBounceSubscribersRequest(this).toQueryParamRequest();
    }

    @Override
    public Set<String> getFields() {
        return this.fields;
    }

    @Override
    public String getPathId() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public BounceQueryBuilder addVisibleFields(Collection<String> fields) throws NullPointerException {
        Objects.requireNonNull(fields, "Parameter fields is null.");
        this.getFields().addAll(fields);
        return this;
    }

    @Override
    public BounceQueryBuilder addVisibleField(String field) throws NullPointerException {
        Objects.requireNonNull(field, "Parameter field is null.");
        this.getFields().add(field);
        return this;
    }

    public static final class QueryBounceSubscribersRequest extends PaginatedRequest {
        private final BounceType type;
        private final BounceReason reason;
        private final Set<String> fields;

        public QueryBounceSubscribersRequest(BounceQueryBuilder builder) {
            this.type = builder.type;
            this.reason = builder.reason;
            this.fields = builder.fields;
            this.size = builder.size;
            this.page = builder.page;
            this.sortField = builder.sortField;
            this.sortDir = builder.sortDir;
        }

        @Override
        public String toQueryParamRequest() {
            List<NameValuePair> queryParams = new ArrayList<>();
            addQueryParams(queryParams);
            if (type != null) {
                queryParams.add(new BasicNameValuePair("type", type.name()));
            }
            if (reason != null) {
                queryParams.add(new BasicNameValuePair("reason", reason.getValue()));
            }
            if (fields != null && !fields.isEmpty()) {
                queryParams.add(new BasicNameValuePair("fields", fields.stream().collect(Collectors.joining(","))));
            }
            return new URIBuilder().setParameters(queryParams).toString();
        }
    }
}
