/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.model.request;

import com.google.gson.Gson;
import com.apifon.sdk.resource.helpers.IJsonSerializable;
import com.apifon.sdk.resource.helpers.IHttpEtag;

/**
 *
 * @author Apifon
 */
public class GdprFieldRequestBuilder {

    private final String listId;
    private String field;
    private String name;
    private String description;
    private String etag;

    private GdprFieldRequestBuilder(String name, String listId) {
        this.name = name;
        this.listId = listId;
    }
    
    public static GdprFieldRequestBuilder getUpdateGdprFieldRequestBuilder(String name, String listId, String field) {
        return new GdprFieldRequestBuilder(name, listId).withField(field);
    }

    public static GdprFieldRequestBuilder getAddGdprFieldRequestBuilder(String name, String listId) {
        return new GdprFieldRequestBuilder(name, listId);
    }

    /**
     * Sets GDPR field description
     * @param description
     * @return this for chaining
     */
    public GdprFieldRequestBuilder withDescription(String description) {
        this.description = description;
        return this;
    }

    /**
     * Force list etag check.
     * @param etag the list etag
     * @return this for chaining
     */
    public GdprFieldRequestBuilder withEtag(String etag) {
        this.etag = etag;
        return this;
    }
    
    /**
     * Set name of GDPR field.
     * @param name
     * @return this for chaining.
     */
    public GdprFieldRequestBuilder withName(String name) {
        this.name = name;
        return this;
    }
    
    /**
     * Sets the field (Unique identifier) of the GDPR field.<br>
     * <b>Note: This is only used for update on GDPR field, it's ignored otherwise.</b>
     * @param field
     * @return 
     */
    public GdprFieldRequestBuilder withField(String field) {
        this.field = field;
        return this;
    }

    /**
     * Build and return the GdprFieldRequest
     * @return 
     */
    public GdprFieldRequest build() {
        return new GdprFieldRequest(this);
    }

    public static final class GdprFieldRequest implements IJsonSerializable, IHttpEtag {

        private final transient String etag;
        private final transient String listId;
        private final transient String field;
        private final String name;
        private final String description;

        public GdprFieldRequest(GdprFieldRequestBuilder builder) {
            this.name = builder.name;
            this.description = builder.description;
            this.etag = builder.etag;
            this.listId = builder.listId;
            this.field = builder.field;
        }

        public String getName() {
            return name;
        }

        public String getDescription() {
            return description;
        }

        public String getListId() {
            return listId;
        }

        public String getField() {
            return field;
        }

        @Override
        public String getBody() {
            return new Gson().toJson(this);
        }

        @Override
        public String getEtag() {
            return this.etag;
        }

    }
}
