/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.model.request;

import com.apifon.sdk.resource.helpers.PaginatedRequest;
import com.apifon.sdk.resource.helpers.PaginatedRequestBuilder;
import com.apifon.sdk.resource.helpers.ListType;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.message.BasicNameValuePair;
import com.apifon.sdk.resource.helpers.IFieldSerializable;
import java.util.Collection;

/**
 *
 * @author Apifon
 */
public class ListQueryBuilder extends PaginatedRequestBuilder implements IFieldSerializable<ListQueryBuilder> {

    private ListType type;
    private final Set<String> fields;

    private ListQueryBuilder() {
        this.fields = new HashSet<>();
    }

    public static ListQueryBuilder getBuilder() {
        return new ListQueryBuilder();
    }

    public ListQueryBuilder withType(ListType type) throws NullPointerException {
        Objects.requireNonNull(type, "Parameter type is null.");
        this.type = type;
        return this;
    }

    @Override
    public String toQueryParamRequest() {
        return new ListQueryRequest(this).toQueryParamRequest();
    }

    @Override
    public PaginatedRequest build() {
        return new ListQueryRequest(this);
    }

    @Override
    public Set<String> getFields() {
        return this.fields;
    }

    @Override
    public String getPathId() {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public ListQueryBuilder addVisibleFields(Collection<String> fields) throws NullPointerException {
        Objects.requireNonNull(fields, "Parameter fields is null.");
        this.getFields().addAll(fields);
        return this;
    }

    @Override
    public ListQueryBuilder addVisibleField(String field) throws NullPointerException {
        Objects.requireNonNull(field, "Parameter field is null.");
        this.getFields().add(field);
        return this;
    }

    public static final class ListQueryRequest extends PaginatedRequest {

        private final ListType type;
        private final Set<String> fields;

        public ListQueryRequest(ListQueryBuilder builder) {
            this.type = builder.type;
            this.fields = builder.fields;
            this.size = builder.size;
            this.page = builder.page;
            this.sortField = builder.sortField;
            this.sortDir = builder.sortDir;
        }

        @Override
        public String toQueryParamRequest() {
            List<NameValuePair> queryParams = new ArrayList<>();
            addQueryParams(queryParams);
            if (type != null) {
                queryParams.add(new BasicNameValuePair("type", type.name()));
            }
            if (fields != null && !fields.isEmpty()) {
                queryParams.add(new BasicNameValuePair("fields", fields.stream().collect(Collectors.joining(","))));
            }
            return new URIBuilder().setParameters(queryParams).toString();
        }
    }
}
