/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.model.request;

import com.apifon.sdk.resource.helpers.ListType;
import com.apifon.sdk.response.ListResponse;
import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import java.util.Objects;
import com.apifon.sdk.resource.helpers.IJsonSerializable;
import com.apifon.sdk.resource.helpers.IHttpEtag;

/**
 *
 * @author Apifon
 */
public class ListRequestBuilder implements IHttpEtag {

    private transient final String id;
    private final ListType type;
    private String title;
    private String prefix;
    private GdprPropertiesBuilder.GdprProperties gdprProperties;
    private CampaignDefaultsBuilder.CampaignDefaults campaignDefaults;
    private String description;
    private String etag;

    private ListRequestBuilder(ListType type, String title, String prefix, String listId) {
        this.type = type;
        this.title = title;
        this.prefix = prefix;
        this.id = listId;
    }

    private ListRequestBuilder(String listId) {
        this.type = null;
        this.id = listId;
    }

    /**
     * Constructs a ListRequestBuilder with fields set directly from the API
     * response.<br>
     * <b>Note: This will use the etag to confirm the update on the list</b>
     *
     * @param list
     * @return
     */
    public static ListRequestBuilder getUpdateListRequestBuilder(ListResponse list) throws NullPointerException {
        Objects.requireNonNull(list, "Parameter list is null.");
        Objects.requireNonNull(list.getId(), "Parameter list.id is null.");
        return new ListRequestBuilder(null, list.getTitle(), list.getPrefix(), list.getId())
                .withCampaignDefaults(list.getCampaignDefaults())
                .withGdprProperties(list.getGdprProperties())
                .withDescription(list.getDescription())
                .withEtag(list.getEtag());
    }

    /**
     * Constructs a ListRequestBuilder for update requests with the id
     * given.<br>
     *
     * @param listId the list ID. May NOT be null.
     * @return
     * @throws NullPointerException if listId is null.
     */
    public static ListRequestBuilder getUpdateListRequestBuilder(String listId) throws NullPointerException {
        Objects.requireNonNull(listId, "Parameter listId is null.");
        return new ListRequestBuilder(listId);
    }

    /**
     * Constructs a ListRequestBuilder for add requests with the required
     * fields.<br>
     *
     * @param type the list Type. May NOT be null.
     * @param title the title of the list. May NOT be null.
     * @param prefix the prefix. May NOT be null.
     * @return
     * @throws NullPointerException if any parameter is null.
     */
    public static ListRequestBuilder getAddListRequestBuilder(ListType type, String title, String prefix) throws NullPointerException {
        Objects.requireNonNull(type, "Parameter type is null.");
        Objects.requireNonNull(title, "Parameter title is null.");
        Objects.requireNonNull(prefix, "Parameter prefix is null.");
        return new ListRequestBuilder(type, title, prefix, null);
    }

    /**
     * Sets the list title. May NOT be null.
     *
     * @param title
     * @return this for chaining.
     * @throws NullPointerException if title is null.
     */
    public ListRequestBuilder withTitle(String title) throws NullPointerException {
        Objects.requireNonNull(title, "Parameter title is null.");
        this.title = title;
        return this;
    }

    /**
     * Sets the list prefix. May NOT be null.
     *
     * @param prefix
     * @return this for chaining.
     * @throws NullPointerException if prefix is null.
     */
    public ListRequestBuilder withPrefix(String prefix) throws NullPointerException {
        Objects.requireNonNull(prefix, "Parameter prefix is null.");
        this.prefix = prefix;
        return this;
    }

    /**
     * Sets the list gdpr_properties from a GdprPropertiesBuilder by calling the
     * build() method.
     *
     * @param builder
     * @return this for chaining.
     * @see GdprPropertiesBuilder
     */
    public ListRequestBuilder withGdprProperties(GdprPropertiesBuilder builder) {
        this.gdprProperties = builder.build();
        return this;
    }

    /**
     * Sets the list gdpr_properties from a GdprProperties entity.
     *
     * @param gdprProperties
     * @return this for chaining.
     * @see GdprPropertiesBuilder
     */
    public ListRequestBuilder withGdprProperties(GdprPropertiesBuilder.GdprProperties gdprProperties) {
        this.gdprProperties = gdprProperties;
        return this;
    }

    /**
     * Sets the list campaign_default from a CampaignDefaultsBuilder by calling
     * the build() method.
     *
     * @param builder
     * @return this for chaining.
     * @see CampaignDefaultsBuilder
     */
    public ListRequestBuilder withCampaignDefaults(CampaignDefaultsBuilder builder) {
        this.campaignDefaults = builder.build();
        return this;
    }

    /**
     * Sets the list campaign_defaults from a CampaignDefaults entity.
     *
     * @param campaignDefaults
     * @return this for chaining.
     * @see CampaignDefaultsBuilder
     */
    public ListRequestBuilder withCampaignDefaults(CampaignDefaultsBuilder.CampaignDefaults campaignDefaults) {
        this.campaignDefaults = campaignDefaults;
        return this;
    }

    /**
     * Sets the list description.
     *
     * @param description
     * @return this for chaining.
     */
    public ListRequestBuilder withDescription(String description) {
        this.description = description;
        return this;
    }

    /**
     * Sets the list etag value<br><br>
     * <b>Note: If set the API will try to evaluate the ETag precondition,
     * resulting in an exception if there is a missmatch.</b>
     *
     * @param etag
     * @return this for chaining
     */
    public ListRequestBuilder withEtag(String etag) {
        this.etag = etag;
        return this;
    }

    /**
     * @return Returns a ListRequest to be dispatched to the API.
     * @see ListRequest
     */
    public ListRequest build() {
        return new ListRequest(this);
    }

    @Override
    public String getEtag() {
        return this.etag;
    }

    public static final class ListRequest implements IJsonSerializable, IHttpEtag {

        private transient final String id;
        private transient final String etag;
        @SerializedName("gdpr_properties")
        private final GdprPropertiesBuilder.GdprProperties gdprProperties;
        @SerializedName("campaign_defaults")
        private final CampaignDefaultsBuilder.CampaignDefaults campaignDefaults;
        private final ListType type;
        private final String title;
        private final String prefix;
        private final String description;

        public ListRequest(ListRequestBuilder builder) {
            this.gdprProperties = builder.gdprProperties;
            this.campaignDefaults = builder.campaignDefaults;
            this.type = builder.type;
            this.title = builder.title;
            this.prefix = builder.prefix;
            this.description = builder.description;
            this.id = builder.id;
            this.etag = builder.getEtag();
        }

        public GdprPropertiesBuilder.GdprProperties getGdprProperties() {
            return gdprProperties;
        }

        public CampaignDefaultsBuilder.CampaignDefaults getCampaignDefaults() {
            return campaignDefaults;
        }

        public ListType getType() {
            return type;
        }

        public String getTitle() {
            return title;
        }

        public String getPrefix() {
            return prefix;
        }

        public String getDescription() {
            return description;
        }

        public String getId() {
            return id;
        }

        @Override
        public String getBody() {
            return new Gson().toJson(this);
        }

        @Override
        public String getEtag() {
            return this.etag;
        }
    }
}
