/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.model.request;

import com.apifon.sdk.model.Sort;
import com.apifon.sdk.resource.helpers.Condition;
import com.apifon.sdk.resource.helpers.ConditionOperator;
import com.apifon.sdk.resource.helpers.IFieldSerializable;
import com.apifon.sdk.resource.helpers.IJsonSerializable;
import com.apifon.sdk.resource.helpers.SortDirection;
import com.google.gson.Gson;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;

/**
 *
 * @author Apifon
 */
public class SubscriberConditionQueryRequestBuilder implements IFieldSerializable<SubscriberConditionQueryRequestBuilder> {

    private final String listId;
    private int page;
    private int size;
    private final List<Condition> conditions;
    private final Set<String> fields;
    private Sort sort;

    private SubscriberConditionQueryRequestBuilder(String listId) {
        this.listId = listId;
        this.page = 1;
        this.size = 200;
        this.conditions = new ArrayList<>();
        this.fields = new HashSet<>();
        this.sort = Sort.getDefaultSort();
    }

    public static SubscriberConditionQueryRequestBuilder getSubscriberConditionsBuilder(String listId) {
        return new SubscriberConditionQueryRequestBuilder(listId);
    }

    public SubscriberConditionQueryRequestBuilder clearConditions() {
        this.conditions.clear();
        return this;
    }
    
    public SubscriberConditionQueryRequestBuilder withCondition(String name, ConditionOperator operator, Object value) {
        this.conditions.add(new Condition(name, operator, value));
        return this;
    }

    public SubscriberConditionQueryRequestBuilder withCondition(Condition condition) {
        this.conditions.add(condition);
        return this;
    }

    public SubscriberConditionQueryRequestBuilder withConditions(Collection<Condition> conditions) {
        this.conditions.addAll(conditions);
        return this;
    }

    public SubscriberConditionQueryRequestBuilder withPage(int page) {
        this.page = page;
        return this;
    }

    public SubscriberConditionQueryRequestBuilder withSize(int size) {
        this.size = size;
        return this;
    }

    public SubscriberConditionQueryRequestBuilder withSort(Sort sort) {
        this.sort = sort;
        return this;
    }

    public SubscriberConditionQueryRequestBuilder withSort(SortDirection sortDirection, String sortField) {
        this.sort = new Sort(sortDirection, sortField);
        return this;
    }
    
    public SubscriberConditionQueryRequest build() {
        return new SubscriberConditionQueryRequest(this);
    }

    @Override
    public Set<String> getFields() {
        return this.fields;
    }

    @Override
    public String getPathId() {
        return this.listId;
    }

    @Override
    public SubscriberConditionQueryRequestBuilder addVisibleFields(Collection<String> fields) throws NullPointerException {
        Objects.requireNonNull(fields, "Parameter fields is null.");
        this.getFields().addAll(fields);
        return this;
    }

    @Override
    public SubscriberConditionQueryRequestBuilder addVisibleField(String field) throws NullPointerException {
        Objects.requireNonNull(field, "Parameter field is null.");
        this.getFields().add(field);
        return this;
    }

    public static final class SubscriberConditionQueryRequest implements IJsonSerializable {

        private final transient String listId;
        private final int page;
        private final int size;
        private final List<Condition> conditions;
        private final Set<String> fields;
        private final Sort sort;

        public SubscriberConditionQueryRequest(SubscriberConditionQueryRequestBuilder builder) {
            this.listId = builder.listId;
            this.page = builder.page;
            this.size = builder.size;
            this.conditions = builder.conditions;
            this.fields = builder.fields;
            this.sort = builder.sort;
        }

        public String getListId() {
            return listId;
        }

        public int getPage() {
            return page;
        }

        public int getSize() {
            return size;
        }

        public List<Condition> getConditions() {
            return Collections.unmodifiableList(conditions);
        }

        public Set<String> getFields() {
            return Collections.unmodifiableSet(fields);
        }

        public Sort getSort() {
            return sort;
        }

        @Override
        public String getBody() {
            return new Gson().toJson(this);
        }
    }
}
