/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.model.request;

import com.apifon.sdk.resource.helpers.SubscriberFieldDateFormat;
import com.apifon.sdk.resource.helpers.SubscriberFieldType;
import com.apifon.sdk.response.SubscriberFieldResponse;
import com.apifon.sdk.serializer.SubscriberFieldDateFormatAdapter;
import com.google.gson.Gson;
import com.google.gson.annotations.JsonAdapter;
import com.google.gson.annotations.SerializedName;
import java.util.Objects;
import com.apifon.sdk.resource.helpers.IJsonSerializable;
import com.apifon.sdk.resource.helpers.IHttpEtag;

/**
 *
 * @author Apifon
 */
public class SubscriberFieldRequestBuilder implements IHttpEtag {

    private final transient String listId;
    private transient String field;
    private transient String etag;
    private int length;
    private String name;
    private SubscriberFieldType type;
    private SubscriberFieldDateFormat dateFormat;
    private boolean internal;

    private SubscriberFieldRequestBuilder(String listId, String field) {
        this.field = field;
        this.listId = listId;
    }

    private SubscriberFieldRequestBuilder(String listId, SubscriberFieldType type, int length, String name) {
        this.type = type;
        this.length = length;
        this.name = name;
        this.internal = false;
        this.listId = listId;
    }

    public static SubscriberFieldRequestBuilder getUpdateSubscriberFieldRequestBuilder(String listId, String field, String name, int length) throws NullPointerException {
        Objects.requireNonNull(field, "Parameter field is null.");
        return new SubscriberFieldRequestBuilder(listId, field)
                .withName(name)
                .withLength(length);
    }

    public static SubscriberFieldRequestBuilder getUpdateSubscriberFieldRequestBuilder(String listId, SubscriberFieldResponse subscriberField) throws NullPointerException {
        return new SubscriberFieldRequestBuilder(listId, subscriberField.getField())
                .withName(subscriberField.getName())
                .withLength(subscriberField.getLength())
                .withDateFormat(subscriberField.getDateFormat())
                .withInternal(subscriberField.isInternal());
    }

    public static SubscriberFieldRequestBuilder getAddRequestBuilder(String listId, SubscriberFieldType type, int length, String name) {
        return new SubscriberFieldRequestBuilder(listId, type, length, name);
    }

    public SubscriberFieldRequestBuilder withDateFormat(SubscriberFieldDateFormat dateFormat) {
        this.dateFormat = dateFormat;
        return this;
    }

    public SubscriberFieldRequestBuilder withInternal(boolean internal) {
        this.internal = internal;
        return this;
    }

    public SubscriberFieldRequestBuilder withName(String name) {
        this.name = name;
        return this;
    }

    public SubscriberFieldRequestBuilder withLength(int length) {
        this.length = length;
        return this;
    }
    
    public SubscriberFieldRequestBuilder withListEtag(String listEtag) {
        this.etag = listEtag;
        return this;
    }
    
    public SubscriberFieldRequest build() {
        return new SubscriberFieldRequest(this);
    }

    @Override
    public String getEtag() {
        return this.etag;
    }

    public static final class SubscriberFieldRequest implements IJsonSerializable, IHttpEtag {
        private final transient String listId;
        private final transient String field;
        private final transient String etag;
        private final int length;
        private final String name;
        private final SubscriberFieldType type;
        @JsonAdapter(SubscriberFieldDateFormatAdapter.class)
        @SerializedName("date_format")
        private final SubscriberFieldDateFormat dateFormat;
        private final boolean internal;
        
        public SubscriberFieldRequest(SubscriberFieldRequestBuilder builder) {
            this.field = builder.field;
            this.length = builder.length;
            this.name = builder.name;
            this.type = builder.type;
            this.dateFormat = builder.dateFormat;
            this.internal = builder.internal;
            this.listId = builder.listId;
            this.etag = builder.etag;
        }

        public String getField() {
            return field;
        }

        public int getLength() {
            return length;
        }

        public String getName() {
            return name;
        }

        public SubscriberFieldType getType() {
            return type;
        }

        public SubscriberFieldDateFormat getDateFormat() {
            return dateFormat;
        }

        public boolean isInternal() {
            return internal;
        }

        public String getListId() {
            return listId;
        }

        @Override
        public String getBody() {
            return new Gson().toJson(this);
        }

        @Override
        public String getEtag() {
            return this.etag;
        }
    }
}
