/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.model.request;

import com.apifon.sdk.resource.helpers.IFieldSerializable;
import com.apifon.sdk.resource.helpers.PaginatedRequest;
import com.apifon.sdk.resource.helpers.PaginatedRequestBuilder;
import com.apifon.sdk.resource.helpers.SubscriberStatus;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;
import org.apache.http.NameValuePair;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.message.BasicNameValuePair;

/**
 *
 * @author Apifon
 */
public class SubscriberQueryBuilder extends PaginatedRequestBuilder implements IFieldSerializable<SubscriberQueryBuilder> {

    private final String listId;
    private SubscriberStatus status;
    private SubscriberStatus emailStatus;
    private Set<String> fields;

    private SubscriberQueryBuilder(String listId) {
        this.listId = listId;
    }

    public static SubscriberQueryBuilder getQueryBuilder(String listId) {
        return new SubscriberQueryBuilder(listId);
    }

    public SubscriberQueryBuilder withStatus(SubscriberStatus status) {
        this.status = status;
        return this;
    }

    public SubscriberQueryBuilder withEmailStatus(SubscriberStatus emailStatus) {
        this.emailStatus = emailStatus;
        return this;
    }

    @Override
    public PaginatedRequest build() {
        return new SubscriberQueryRequest(this);
    }

    @Override
    public String toQueryParamRequest() {
        return build().toQueryParamRequest();
    }

    @Override
    public Set<String> getFields() {
        return this.fields;
    }

    @Override
    public String getPathId() {
        return this.listId;
    }

    @Override
    public SubscriberQueryBuilder addVisibleFields(Collection<String> fields) throws NullPointerException {
        Objects.requireNonNull(fields, "Parameter fields is null.");
        this.getFields().addAll(fields);
        return this;
    }

    @Override
    public SubscriberQueryBuilder addVisibleField(String field) throws NullPointerException {
        Objects.requireNonNull(field, "Parameter field is null.");
        this.getFields().add(field);
        return this;
    }

    public static final class SubscriberQueryRequest extends PaginatedRequest {

        private final String listId;
        private final SubscriberStatus status;
        private final SubscriberStatus emailStatus;
        private final Set<String> fields;
        
        public SubscriberQueryRequest(SubscriberQueryBuilder builder) {
            this.listId = builder.listId;
            this.status = builder.status;
            this.emailStatus = builder.emailStatus;
            this.fields = builder.fields;
            this.page = builder.page;
            this.size = builder.size;
            this.sortDir = builder.sortDir;
            this.sortField = builder.sortField;
        }
        
        @Override
        public String getPathId() {
            return this.listId;
        }

        @Override
        public String toQueryParamRequest() {
            List<NameValuePair> queryParams = new ArrayList<>();
            addQueryParams(queryParams);
            if (status != null) {
                queryParams.add(new BasicNameValuePair("status", status.name()));
            }
            if (emailStatus != null) {
                queryParams.add(new BasicNameValuePair("email_status", emailStatus.name()));
            }
            if (fields != null && !fields.isEmpty()) {
                queryParams.add(new BasicNameValuePair("fields", fields.stream().collect(Collectors.joining(","))));
            }
            return new URIBuilder().setParameters(queryParams).toString();
        }
    }

}
