/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.model.request;

import com.apifon.sdk.model.SubscriberGdprField;
import com.apifon.sdk.resource.helpers.IJsonSerializable;
import com.apifon.sdk.resource.helpers.SubscriberStatus;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.apifon.sdk.resource.helpers.IHttpEtag;
import com.apifon.sdk.response.SubscriberResponse;
import com.google.gson.Gson;
import com.google.gson.annotations.SerializedName;
import java.util.Collections;

/**
 *
 * @author Apifon
 */
public class SubscriberRequestBuilder implements IHttpEtag {

    private final String listId;
    private final Map<String, Object> subscriberFields;
    private final Map<String, SubscriberGdprField> gdprFields;
    private String etag;
    private String destination;
    private SubscriberStatus status;
    private String email;
    private SubscriberStatus emailStatus;
    private String lookupMD5;
    private List<String> groupOptions;

    private SubscriberRequestBuilder(String listId) {
        this.listId = listId;
        this.subscriberFields = new HashMap<>();
        this.gdprFields = new HashMap<>();
    }
    
    public static SubscriberRequestBuilder getBuilderFromSubscriber(SubscriberResponse subscriber) {
        return new SubscriberRequestBuilder(subscriber.getListId())
                .withDestination(subscriber.getDestination(), subscriber.getStatus())
                .withEmail(subscriber.getEmail(), subscriber.getEmailStatus())
                .withSubscriberFields(subscriber.getSubscriberFields())
                .withGdprFields(subscriber.getGdprFields())
                .withLookupMD5(subscriber.getDestinationId() == null ? subscriber.getEmailId() : subscriber.getDestinationId());
    }

    public static SubscriberRequestBuilder getAddSubscriberRequestBuilder(String listId) {
        return new SubscriberRequestBuilder(listId);
    }

    public SubscriberRequestBuilder withLookupMD5(String lookupMD5) {
        this.lookupMD5 = lookupMD5;
        return this;
    }
    
    public SubscriberRequestBuilder withDestination(String destination, SubscriberStatus status) {
        this.destination = destination;
        this.status = status;
        return this;
    }

    public SubscriberRequestBuilder withEmail(String email, SubscriberStatus emailStatus) {
        this.email = email;
        this.emailStatus = emailStatus;
        return this;
    }

    public SubscriberRequestBuilder withStatus(SubscriberStatus status) {
        this.status = status;
        return this;
    }

    public SubscriberRequestBuilder withEmailStatus(SubscriberStatus emailStatus) {
        this.emailStatus = emailStatus;
        return this;
    }

    public SubscriberRequestBuilder withEtag(String etag) {
        this.etag = etag;
        return this;
    }
    
    public SubscriberRequestBuilder clearSubscriberFields() {
        this.subscriberFields.clear();
        return this;
    }

    public SubscriberRequestBuilder withSubscriberField(String field, Object value) {
        this.subscriberFields.put(field, value);
        return this;
    }
    
    public SubscriberRequestBuilder withSubscriberFields(Map<String, Object> subscriberFields) {
        if(subscriberFields != null && !subscriberFields.isEmpty()){
            this.subscriberFields.putAll(subscriberFields);
        }
        return this;
    }
    
    public SubscriberRequestBuilder clearGdprFields() {
        this.gdprFields.clear();
        return this;
    }

    public SubscriberRequestBuilder withGdprField(String field, boolean gdprCompliance) {
        if (this.gdprFields.containsKey(field)) {
            this.gdprFields.get(field).setMobile(gdprCompliance);
            this.gdprFields.get(field).setEmail(gdprCompliance);
        } else {
            SubscriberGdprField newGdprField = new SubscriberGdprField();
            newGdprField.setMobile(gdprCompliance);
            newGdprField.setEmail(gdprCompliance);
            this.gdprFields.put(field, newGdprField);
        }
        return this;
    }

    public SubscriberRequestBuilder withMobileGdprField(String field, boolean gdprCompliance) {
        if (this.gdprFields.containsKey(field)) {
            this.gdprFields.get(field).setMobile(gdprCompliance);
        } else {
            SubscriberGdprField newGdprField = new SubscriberGdprField();
            newGdprField.setMobile(gdprCompliance);
            this.gdprFields.put(field, newGdprField);
        }
        return this;
    }

    public SubscriberRequestBuilder withEmailGdprField(String field, boolean gdprCompliance) {
        if (this.gdprFields.containsKey(field)) {
            this.gdprFields.get(field).setEmail(gdprCompliance);
        } else {
            SubscriberGdprField newGdprField = new SubscriberGdprField();
            newGdprField.setEmail(gdprCompliance);
            this.gdprFields.put(field, newGdprField);
        }
        return this;
    }
    
    public SubscriberRequestBuilder withGdprFields(Map<String, SubscriberGdprField> gdprFields) {
        if(gdprFields != null && !gdprFields.isEmpty()){
            this.gdprFields.putAll(gdprFields);
        }
        return this;
    }

    public SubscriberRequestBuilder withGroupOptions(List<String> groupOptions){
        this.groupOptions = groupOptions;
        return this;
    }
    
    public SubscriberRequest build() {
        return new SubscriberRequest(this);
    }

    @Override
    public String getEtag() {
        return this.etag;
    }

    public static final class SubscriberRequest implements IJsonSerializable, IHttpEtag {

        private final transient String listId;
        private final transient String etag;
        private final transient String lookupMD5;
        private final String destination;
        private final SubscriberStatus status;
        private final String email;
        @SerializedName("email_status")
        private final SubscriberStatus emailStatus;
        @SerializedName("subscriber_fields")
        private final Map<String, Object> subscriberFields;
        @SerializedName("gdpr_fields")
        private final Map<String, SubscriberGdprField> gdprFields;
        private final List<String> groupOptions;

        public SubscriberRequest(SubscriberRequestBuilder builder) {
            this.listId = builder.listId;
            this.etag = builder.etag;
            this.destination = builder.destination;
            this.status = builder.status;
            this.email = builder.email;
            this.emailStatus = builder.emailStatus;
            this.subscriberFields = builder.subscriberFields;
            this.gdprFields = builder.gdprFields;
            this.lookupMD5 = builder.lookupMD5;
            this.groupOptions = builder.groupOptions;
        }

        public String getListId() {
            return listId;
        }

        public String getDestination() {
            return destination;
        }

        public SubscriberStatus getStatus() {
            return status;
        }

        public String getEmail() {
            return email;
        }

        public SubscriberStatus getEmailStatus() {
            return emailStatus;
        }

        public Map<String, Object> getSubscriberFields() {
            return Collections.unmodifiableMap(subscriberFields);
        }

        public Map<String, SubscriberGdprField> getGdprFields() {
            return Collections.unmodifiableMap(gdprFields);
        }

        public String getLookupMD5() {
            return lookupMD5;
        }

        public List<String> getGroupOptions(){ return groupOptions;}

        @Override
        public String getEtag() {
            return this.etag;
        }

        @Override
        public String getBody() {
            return new Gson().toJson(this);
        }
    }
}
