package com.apifon.sdk.resource;

import com.apifon.sdk.Mookee;
import com.apifon.sdk.exception.ApifonRestException;
import com.apifon.sdk.model.IRequest;
import java.util.HashMap;
import java.util.Map;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.message.BasicHeader;

public abstract class AbstractResource {

    private transient String endpoint;
    private transient String curEndpoint;
    private transient String curBody;
    private transient String curMethod;
    private final transient Map<String, String> curHeaders;
    


    public AbstractResource(String endpoint) {
        this.endpoint = endpoint;
        this.curHeaders = new HashMap<>();
    }

    /**
     * @return the endpoint
     */
    public String getEndpoint() {
        return endpoint;
    }

    /**
     * @param endpoint the endpoint to set
     */
    public void setEndpoint(String endpoint) {
        this.endpoint = endpoint;
    }

    /**
     * @return the curEndpoint
     */
    public String getCurEndpoint() {
        return curEndpoint;
    }

    /**
     * @param curEndpoint the curEndpoint to set
     */
    public void setCurEndpoint(String curEndpoint) {
        this.curEndpoint = curEndpoint;
    }

    /**
     * @return the curBody
     */
    public String getCurBody() {
        return curBody;
    }

    /**
     * @param curBody the curBody to set
     */
    public void setCurBody(String curBody) {
        this.curBody = curBody;
    }

    /**
     * @return the curMethod
     */
    public String getCurMethod() {
        return curMethod;
    }

    /**
     * @param curMethod the curMethod to set
     */
    public void setCurMethod(String curMethod) {
        this.curMethod = curMethod;
    }

    /**
     * @return the curHeaders
     */
    public Map<String, String> getCurHeaders() {
        return curHeaders;
    }
    
    public void addEtagHeader(String etag) {
        if (etag == null) {
            return;
        }
        this.curHeaders.put("If-Match", etag);
    }

    public String dispatch() {
        return Mookee.dispatch(this);
    }
    
    public HttpResponse dispatchToResponse() throws ApifonRestException {
        return Mookee.dispatchToResponse(this);
    }
    
  
    
    /**
     * Sends a Create Request.
     * @param request Our Request.
     * @return A response to our request.
     */
    public String create(IRequest request) {
        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/create");
        this.setCurBody(request.getCreateBody());

        return Mookee.dispatch(this);
    }
    
    /**
     * Sends a Read Request.
     * @param request Our Request.
     * @return A response to our request.
     */
    public String read(IRequest request) {
        this.setCurMethod("GET");
        this.setCurEndpoint(this.getEndpoint() + "/read");
        this.setCurBody(request.getReadBody());

        return Mookee.dispatch(this);
    }
    
    /**
     * Sends a Update Request.
     * @param request Our Request.
     * @return A response to our request.
     */
    public String update(IRequest request) {
        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/update");
        this.setCurBody(request.getUpdateBody());

        return Mookee.dispatch(this);
    }
    
    /**
     * Sends a Delete Request.
     * @param request Our Request.
     * @return A response to our request.
     */
    public String delete(IRequest request) {
        this.setCurMethod("DELETE");
        this.setCurEndpoint(this.getEndpoint() + "/delete");
        this.setCurBody(request.getDeleteBody());

        return Mookee.dispatch(this);
    }
    
    public void addHeaders(HttpRequestBase base) {
       for (Map.Entry<String, String> pair : curHeaders.entrySet()) {
            base.addHeader(new BasicHeader(pair.getKey(), pair.getValue()));
        }
    }
    
    public boolean isSuccessful(HttpResponse resp) {
        if(resp.getStatusLine().getStatusCode() == HttpStatus.SC_NOT_MODIFIED){
            System.out.println("Status: 304 " + System.lineSeparator() 
                    + "Title: Not modified. The request was completed but the resource was not modified.");
        }
        return true;
    }
    
}
