/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.resource;

import com.apifon.sdk.exception.ApifonRestException;
import com.apifon.sdk.model.CollectionEntityResponse;
import com.apifon.sdk.model.request.BounceRequestBuilder.BounceRequest;
import com.apifon.sdk.model.request.BounceSingleQuery;
import com.apifon.sdk.resource.helpers.PaginatedRequest;
import com.apifon.sdk.response.BounceResponse;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import java.io.IOException;
import java.lang.reflect.Type;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.util.EntityUtils;

/**
 *
 * @author Apifon
 */
public class BounceListResource extends AbstractResource implements UriLocator<BounceResponse> {

    public BounceListResource() {
        super("/services/api/v1/bounce/subscriber");
    }

    /**
     * This method is used to get a list of bounce subscribers from Apifon REST
     * API.<br><br>
     * Note: This API enforces pagination on results returned.
     *
     * @param request A paginated request
     * @return A CollectionEntityResponse containing a list of BounceResponse along with links for pagination and meta data.
     * @throws ApifonRestException When an API exception is encountered (Invalid
     * request etc)
     */
    public CollectionEntityResponse<BounceResponse> getBounceSubscribers(PaginatedRequest request) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.setCurEndpoint(this.getEndpoint() + "/" + request.toQueryParamRequest());
        this.setCurBody("");
        Type collectionType = new TypeToken<CollectionEntityResponse<BounceResponse>>() {}.getType();
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, collectionType);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }
    
     /**
     * Wrapper method for convenience retrieval of paginated data.
     * @throws java.net.URISyntaxException if the link provided is malformed
     * @see com.apifon.sdk.model.PagingLinks
     * @see getListByResouceUri(URI resourceUri)
     * @param paginationUrl the list link to retrieve
     * @return
     * @throws ApifonRestException 
     */
    public CollectionEntityResponse<BounceResponse> getBounceSubscribers(String paginationUrl) throws ApifonRestException, URISyntaxException {
        return getListByResouceUri(new URI(paginationUrl));
    }

    /**
     * This method is used to retrieve a single bounce subscriber from Apifon
     * REST API.
     *
     * @param request a request describing the parameters of the requested
     * bounce subscriber
     * @return The entity from the API
     * @throws ApifonRestException When an API exception is encountered (Invalid
     * request, Not found etc)
     */
    public BounceResponse getBounceSubscriber(BounceSingleQuery request) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.setCurEndpoint(this.getEndpoint() + "/" + request.getPathId() + request.getQueryParams());
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, BounceResponse.class);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

    /**
     * This method is used to retrieve a single bounce subscriber from Apifon
     * REST API using the given resource URI.
     *
     * @throws ApifonRestException When an API exception is encountered (Invalid
     * request, Not found etc)
     * @see addBounceSubscriber(BounceRequest request) addBounceSubscriber() for resource URI.
     * @param resourceUri the URI of the resource
     * @return The entity from the API
     */
    @Override
    public BounceResponse getByResourceUri(URI resourceUri) throws ApifonRestException {
        String[] pathParams = resourceUri.getPath().split("/");
        String destination = pathParams[pathParams.length - 1];
        return getBounceSubscriber(BounceSingleQuery.getBounceLookup(destination));
    }

    /**
     * This method is used to add a bounce subscriber.
     *
     * @param request A request describing the properties of the bounce
     * subscriber that is to be added.
     * @return A URI describing the resource created.
     * @throws ApifonRestException When an API exception is encountered (Invalid
     * request, Not found etc)
     */
    public URI addBounceSubscriber(BounceRequest request) throws ApifonRestException {
        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/block");
        this.setCurBody(request.getBody());
        HttpResponse response = this.dispatchToResponse();
        return URI.create(response.getFirstHeader(HttpHeaders.LOCATION).getValue());
    }

    /**
     * This method is used to remove a bounce subscriber using Apifon REST API.
     * @param destination The destination that must be removed from the bounce list.
     * @return true if removed, false otherwise.
     * @throws ApifonRestException When an API exception is encountered (Invalid
     * request, Not found etc)
     * @throws NullPointerException if destination is null.
     */
    public boolean removeBounceSubscriber(String destination) throws ApifonRestException, NullPointerException {
        Objects.requireNonNull(destination, "Parameter destination is null.");
        this.setCurMethod("DELETE");
        this.setCurEndpoint(this.getEndpoint() + "/unblock/" + destination);
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        return this.isSuccessful(response);
    }

    @Override
    public CollectionEntityResponse<BounceResponse> getListByResouceUri(URI resourceUri) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.setCurEndpoint(resourceUri.toASCIIString());
        this.setCurBody("");
        Type collectionType = new TypeToken<CollectionEntityResponse<BounceResponse>>() {}.getType();
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, collectionType);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

}
