package com.apifon.sdk.resource;

import com.apifon.sdk.exception.ApifonRestException;
import com.apifon.sdk.model.CollectionEntityResponse;
import com.apifon.sdk.model.request.*;
import com.apifon.sdk.response.GroupResponse;
import com.google.gson.Gson;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;

import java.io.IOException;
import java.net.URI;
import java.util.logging.Level;
import java.util.logging.Logger;

public class GroupResource extends AbstractResource implements UriLocator<GroupResponse> {
    public GroupResource() {
        super("/services/api/v1/list/{list_id}/group");
    }
    /**
     * @param endpoint the curEndpoint to set
     * @param listId the list ID
     */
    public void replacePlaceholderAndSetUrl(String endpoint, String listId) {
        super.setCurEndpoint(endpoint.replace("{list_id}", listId));
    }
    /**
     * Creates a GroupRequest using the Apifon REST API.
     * @param request
     * @return a URI describing the resource created.
     * @throws ApifonRestException When an API exception is encountered
     */
    public URI addGroup(GroupRequestBuilder.GroupRequest request) throws ApifonRestException {
        if(request.getListId() == null || request.getListId().isEmpty()){
            throw new IllegalArgumentException("request.id is not set.");
        }
        this.setCurMethod("POST");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/", request.getListId());
        this.setCurBody(request.getBody());
        HttpResponse response = this.dispatchToResponse();
        return URI.create(response.getFirstHeader(HttpHeaders.LOCATION).getValue());
    }

    /**
     * Updates an existing GroupResponse according to request.
     * @param request
     * @return a URI describing the resource updated.
     * @throws ApifonRestException When an API exception is encountered
     * @throws IllegalArgumentException if the request doesn't contain id.
     */
    public URI updateGroup(GroupRequestBuilder.GroupRequest request) throws ApifonRestException, IllegalArgumentException {
        if (request.getListId() == null || request.getListId().isEmpty()) {
            throw new IllegalArgumentException("request.id is not set.");
        }
        if (request.getGroupField() == null || request.getGroupField().isEmpty()) {
            throw new IllegalArgumentException("request.field is not set.");
        }
        this.setCurMethod("PUT");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + request.getGroupField(),request.getListId());
        this.setCurBody(request.getBody());
        HttpResponse response = this.dispatchToResponse();
        return URI.create(response.getFirstHeader(HttpHeaders.LOCATION).getValue());
    }

    /**
     * Used to get a single GroupSingleQuery from the API.
     * @param request
     * @return
     * @throws ApifonRestException When an API exception is encountered
     */
    public GroupResponse getGroup(GroupSingleQuery request) throws ApifonRestException {
        if(request.getListId() == null || request.getListId().isEmpty()){
            throw new IllegalArgumentException("request.id is not set.");
        }
        if (request.getPathId() == null || request.getPathId().isEmpty()) {
            throw new IllegalArgumentException("request.field is not set.");
        }
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + request.getPathId(),request.getListId());
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, GroupResponse.class);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

    /**
     * Removes the group with the given id <br><br>
     * <b>NOTE: This is a destructive irreversable operation</b>
     *
     * @param listId the list id.
     * @param groupField the group to remove.
     * @return true if removed, false otherwise
     * @throws ApifonRestException When an API exception is encountered
     */
    public boolean removeGroup(String listId, String groupField) throws ApifonRestException {
        this.setCurMethod("DELETE");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + groupField,listId);
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        return this.isSuccessful(response);
    }

    @Override
    public GroupResponse getByResourceUri(URI resourceUri) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.setCurEndpoint(resourceUri.toASCIIString());
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, GroupResponse.class);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

    @Override
    public CollectionEntityResponse<GroupResponse> getListByResouceUri(URI resourceUri) throws ApifonRestException {
        return null;
    }
}
