/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.resource;

import com.apifon.sdk.exception.ApifonRestException;
import com.apifon.sdk.model.CollectionEntityResponse;
import com.apifon.sdk.model.request.ListSingleQuery;
import com.apifon.sdk.model.request.ListRequestBuilder;
import com.apifon.sdk.resource.helpers.PaginatedRequest;
import com.apifon.sdk.response.ListResponse;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import java.io.IOException;
import java.lang.reflect.Type;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpResponse;
import org.apache.http.util.EntityUtils;

/**
 *
 * @author Apifon
 */
public class ListResource extends AbstractResource implements UriLocator<ListResponse> {

    public ListResource() {
        super("/services/api/v1/list");
    }

    /**
     * Creates a ListResponse using the Apifon REST API.
     * @param request
     * @return a URI describing the resource created.
     * @throws ApifonRestException When an API exception is encountered
     */
    public URI createList(ListRequestBuilder.ListRequest request) throws ApifonRestException {
        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/");
        this.setCurBody(request.getBody());
        HttpResponse response = this.dispatchToResponse();
        return URI.create(response.getFirstHeader(HttpHeaders.LOCATION).getValue());
    }

    /**
     * Updates an existing ListResponse according to request.
     * @param request
     * @return a URI describing the resource updated.
     * @throws ApifonRestException When an API exception is encountered
     * @throws IllegalArgumentException if the request doesn't containg id.
     */
    public URI updateList(ListRequestBuilder.ListRequest request) throws ApifonRestException, IllegalArgumentException {
        if (request.getId() == null || request.getId().isEmpty()) {
            throw new IllegalArgumentException("request.id is not set.");
        }
        this.setCurMethod("PUT");
        this.setCurEndpoint(this.getEndpoint() + "/" + request.getId());
        this.setCurBody(request.getBody());
        HttpResponse response = this.dispatchToResponse();
        return URI.create(response.getFirstHeader(HttpHeaders.CONTENT_LOCATION).getValue());
    }

    /**
     * Used to get a single ListResponse from the API.
     * @param request
     * @return 
     * @throws ApifonRestException When an API exception is encountered
     */
    public ListResponse getList(ListSingleQuery request) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.setCurEndpoint(this.getEndpoint() + "/" + request.getPathId() + request.getQueryParams());
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, ListResponse.class);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

      /**
     * This method is used to get a CollectionEntityResponse containing ListResponse from Apifon REST API.<br><br>
     * Note: This API enforces pagination on results returned.
     * @param request A paginated request
     * @return A CollectionEntityResponse containing a list of ListResponse along with links for pagination and meta data.
     * @throws ApifonRestException When an API exception is encountered
     */
    public CollectionEntityResponse<ListResponse> getLists(PaginatedRequest request) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.setCurEndpoint(this.getEndpoint() + "/" + request.toQueryParamRequest());
        this.setCurBody("");
        Type collectionType = new TypeToken<CollectionEntityResponse<ListResponse>>() {
        }.getType();
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, collectionType);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

    /**
     * Wrapper method for convenience retrieval of paginated data.
     *
     * @throws java.net.URISyntaxException if the url provided is malformed
     * @see com.apifon.sdk.model.PagingLinks
     * @see getListByResouceUri(URI resourceUri)
     * @param paginationUrl the list link to retrieve
     * @return
     * @throws ApifonRestException When an API exception is encountered
     */
    public CollectionEntityResponse<ListResponse> getLists(String paginationUrl) throws ApifonRestException, URISyntaxException {
        return getListByResouceUri(new URI(paginationUrl));
    }

    /**
     * Removes the list with the given id and any associated subscribers/landing
     * campaigns.<br><br>
     * <b>NOTE: This is a destructive irreversable operation</b>
     *
     * @param id the list id to remove.
     * @return true if removed, false otherwise
     * @throws ApifonRestException When an API exception is encountered
     */
    public boolean removeList(String id) throws ApifonRestException {
        this.setCurMethod("DELETE");
        this.setCurEndpoint(this.getEndpoint() + "/" + id);
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        return this.isSuccessful(response);
    }

    @Override
    public CollectionEntityResponse<ListResponse> getListByResouceUri(URI resourceUri) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.setCurEndpoint(resourceUri.toASCIIString());
        this.setCurBody("");
        Type collectionType = new TypeToken<CollectionEntityResponse<ListResponse>>() {
        }.getType();
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, collectionType);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

    @Override
    public ListResponse getByResourceUri(URI resourceUri) throws ApifonRestException {
        String[] pathParams = resourceUri.getPath().split("/");
        String listId = pathParams[pathParams.length - 1];
        return getList(ListSingleQuery.getListRequest(listId));
    }

}
