package com.apifon.sdk.resource;

import com.apifon.sdk.response.OtpResponse;
import com.apifon.sdk.model.OtpRequest;
import com.apifon.sdk.response.OtpVerifyResponse;
import com.google.gson.Gson;

/**
 * OTPResource class is used to send OTP Requests to the APIFON Rest API. Call
 * the create() method using an OtpRequest parameter to generate and send an
 * OTP Code or call the verify() method using a reference and an OTP code as 
 * parameters to verify the validity of the provided OTP codes.
 */
public class OTPResource extends AbstractResource {

    public OTPResource() {
        super("/services/api/v1/otp");
    }

    /**
     * This method is used to generate OTP codes and send them through the APIFON Rest API.
     *
     * @param request An OtpRequest.
     * @return The java object that contain all the OTP response that
     * returned from the API call.
     */
    public OtpResponse create(OtpRequest request) {
        Gson gson = new Gson();

        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/create");
        this.setCurBody(request.getCreateBody());
        
        return gson.fromJson(this.dispatch(), OtpResponse.class);
    }

    /**
     * This method is used to generate OTP codes and send them through the APIFON Rest API.
     *
     * @param body The JSon body of the request.
     * @return The java object that contain all the OTP response that
     * returned from the API call.
     */
    public OtpResponse create(String body) {
        Gson gson = new Gson();

        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/create");
        this.setCurBody(body);

        return gson.fromJson(this.dispatch(), OtpResponse.class);
    }

    /**
     * This method is used to verify the validity of an OTP code through the APIFON Rest API.
     *
     * @param reference The reference used to create the OTP.
     * @param otp The OTP provided by the receiver.
     * @return The java object that contain all the gateway response that
     * returned from the API call.
     */
    public OtpVerifyResponse verify(String reference, String otp) {
        Gson gson = new Gson();

        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/verify/" + reference + "/" + otp);
        this.setCurBody(null);

        return gson.fromJson(this.dispatch(), OtpVerifyResponse.class);
    }

}
