package com.apifon.sdk.resource;

import com.apifon.sdk.model.RemoveRequest;
import com.apifon.sdk.model.SmsRequest;
import com.apifon.sdk.response.GatewayResponse;
import com.google.gson.Gson;

/**
 * SMSResource class is used to send SMS Requests to the APIFON Rest API. Call
 * the send() method using an SmsRequest parameter to send SMS Messages or call
 * the remove() method using a RemoveRequest parameter to delete scheduled SMS
 * Requests.
 */
public class SMSResource extends AbstractResource {

    public SMSResource() {
        super("/services/api/v1/sms");
    }

    /**
     * This method is used to send SMS messages through the APIFON Rest API.
     *
     * @param request An SmsRequest.
     * @return The java object that contain all the gateway response that
     * returned from the API call.
     */
    public GatewayResponse send(SmsRequest request) {
        Gson gson = new Gson();

        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/send");
        this.setCurBody(request.getBody());
        String s = this.dispatch();
        return gson.fromJson(s, GatewayResponse.class);
    }

    /**
     * This method is used to send SMS messages through the APIFON Rest API.
     *
     * @param body The JSon body of the request.
     * @return The java object that contain all the gateway response that
     * returned from the API call.
     */
    public GatewayResponse send(String body) {
        Gson gson = new Gson();

        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/send");
        this.setCurBody(body);

        return gson.fromJson(this.dispatch(), GatewayResponse.class);
    }

    /**
     * This method is used to remove SMS messages that are scheduled to be sent.
     *
     * @param request A RemoveRequest.
     * @return an integer that represents the numbed of messages that have been
     * deleted.
     */
    public String remove(RemoveRequest request) {
        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/remove");
        this.setCurBody(request.getBody());

        return this.dispatch();
    }

    /**
     * This method is used to remove SMS messages that are scheduled to be sent.
     *
     * @param body The JSon body of the request.
     * @return an integer that represents the numbed of messages that have been
     * deleted.
     */
    public String remove(String body) {
        this.setCurMethod("POST");
        this.setCurEndpoint(this.getEndpoint() + "/remove");
        this.setCurBody(body);

        return this.dispatch();
    }

}
