/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.resource;

import com.apifon.sdk.exception.ApifonRestException;
import com.apifon.sdk.model.request.SubscriberFieldRequestBuilder;
import java.net.URI;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpResponse;

/**
 *
 * @author Apifon
 */
public class SubscriberFieldResource extends AbstractResource {

    public SubscriberFieldResource() {
        super("/services/api/v1/list/{list_id}/field");
    }

    /**
     * @param endpoint the curEndpoint to set
     * @param listId the list ID
     */
    public void replacePlaceholderAndSetUrl(String endpoint, String listId) {
        super.setCurEndpoint(endpoint.replace("{list_id}", listId));
    }

    /**
     * Adds the given subscriber field.
     *
     * @param request
     * @return The URI associated with the resource created.
     * @throws ApifonRestException if an exception is thrown from the API call
     * (Unparsable request, Resource not found etc)
     */
    public URI addSubscriberField(SubscriberFieldRequestBuilder.SubscriberFieldRequest request) throws ApifonRestException {
        this.setCurMethod("POST");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/", request.getListId());
        this.setCurBody(request.getBody());
        HttpResponse response = this.dispatchToResponse();
        return URI.create(response.getFirstHeader(HttpHeaders.LOCATION).getValue());
    }

    /**
     * Updates a subscriber field on a list according to request.
     *
     * @param request
     * @return The URI associated with the resource updated.
     * @throws ApifonRestException if an exception is thrown from the API call
     * (Unparsable request, Resource not found etc)
     */
    public URI updateSubscriberField(SubscriberFieldRequestBuilder.SubscriberFieldRequest request) throws ApifonRestException {
        this.setCurMethod("PUT");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + request.getField(), request.getListId());
        this.setCurBody(request.getBody());
        HttpResponse response = this.dispatchToResponse();
        return URI.create(response.getFirstHeader(HttpHeaders.LOCATION).getValue());
    }
    
    /**
     * Removes the given subscriber field from the list. <br>
     * This will also delete any values any existing subscribers have for that
     * field.<br>
     * <b>Note: This is a destructive irreversable operation</b>
     *
     * @param listId the ID of the list
     * @param field the field identifier that is to be removed
     * @return true if removed, false otherwise
     * @throws ApifonRestException if an exception is thrown from the API call
     * (Unparsable request, Resource not found etc)
     */
    public boolean removeSubscriberField(String listId, String field) throws ApifonRestException {
        this.setCurMethod("DELETE");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + field, listId);
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        return this.isSuccessful(response);
    }
    
}
