/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.resource;

import com.apifon.sdk.exception.ApifonRestException;
import com.apifon.sdk.model.CollectionEntityResponse;
import com.apifon.sdk.model.request.SubscriberConditionQueryRequestBuilder;
import com.apifon.sdk.model.request.SubscriberConditionQueryRequestBuilder.SubscriberConditionQueryRequest;
import com.apifon.sdk.model.request.SubscriberRequestBuilder;
import com.apifon.sdk.model.request.SubscriberSingleQuery;
import com.apifon.sdk.resource.helpers.PaginatedRequest;
import com.apifon.sdk.response.SubscriberResponse;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import java.io.IOException;
import java.lang.reflect.Type;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHeaders;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.util.EntityUtils;

/**
 *
 * @author Apifon
 */
public class SubscriberResource extends AbstractResource implements UriLocator<SubscriberResponse> {

    public SubscriberResource() {
        super("/services/api/v1/list/{list_id}/subscriber");
    }

    /**
     * @param endpoint the curEndpoint to set
     * @param listId the list ID
     */
    public void replacePlaceholderAndSetUrl(String endpoint, String listId) {
        super.setCurEndpoint(endpoint.replace("{list_id}", listId));
    }

    /**
     * Adds a subscriber according to request
     * @param request
     * @return the URI of the resource created
     * @throws ApifonRestException if an API exception occurs (Invalid request, Not found etc)
     * @throws IllegalArgumentException if the request doesn't contain list ID.
     */
    public URI addSubscriber(SubscriberRequestBuilder.SubscriberRequest request) throws ApifonRestException, IllegalArgumentException {
        if (request.getListId() == null) {
            throw new IllegalArgumentException("request.listId is not provided.");
        }
        this.setCurMethod("POST");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/", request.getListId());
        this.setCurBody(request.getBody());
        HttpResponse response = this.dispatchToResponse();
        return URI.create(response.getFirstHeader(HttpHeaders.LOCATION).getValue());
    }

    /**
     * Updates a subscriber according to request
     * @param request
     * @return the URI of the resource that's updated
     * @throws ApifonRestException if an API exception occurs (Invalid request, Not found etc)
     * @throws IllegalArgumentException if the request doesn't contain list ID or lookupMD5.
     */
    public URI updateSubscriber(SubscriberRequestBuilder.SubscriberRequest request) throws ApifonRestException, IllegalArgumentException {
        if (request.getListId() == null || request.getLookupMD5() == null) {
            throw new IllegalArgumentException("request.listId or request.lookupMD5 is not provided.");
        }
        this.setCurMethod("PUT");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + request.getLookupMD5(), request.getListId());
        this.setCurBody(request.getBody());
        HttpResponse response = this.dispatchToResponse();
        return URI.create(response.getFirstHeader(HttpHeaders.CONTENT_LOCATION).getValue());
    }

    /**
     * Executes a query for a specific subscriber.
     * @param request
     * @return the Subscriber found
     * @throws ApifonRestException if an API exception occurs (Invalid request, Not found etc)
     */
    public SubscriberResponse getSubscriber(SubscriberSingleQuery request) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + request.getPathId() + request.getQueryParams(), request.getListId());
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, SubscriberResponse.class);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

    /**
     * Executes a conditional query for subscribers.<br>
     *  <b>Note: This endpoint returns paginated results.</b>
     * @param request
     * @return A CollectionEntityResponse containing a list of results
     * @throws ApifonRestException if an API exception occurs (Invalid request, Not found etc)
     */
    public CollectionEntityResponse<SubscriberResponse> querySubscribersWithConditions(SubscriberConditionQueryRequest request) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("POST");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/query", request.getListId());
        this.setCurBody(request.getBody());
        Type collectionType = new TypeToken<CollectionEntityResponse<SubscriberResponse>>() {
        }.getType();
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, collectionType);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

    /**
     * Executes a query for subscribers.<br>
     * <b>Note: This endpoint returns paginated results.</b>
     * @param request
     * @return A CollectionEntityResponse containing a list of results
     * @throws ApifonRestException if an API exception occurs (Invalid request, Not found etc) 
     */
    public CollectionEntityResponse<SubscriberResponse> querySubscribers(PaginatedRequest request) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + request.toQueryParamRequest(), request.getPathId());
        this.setCurBody("");
        Type collectionType = new TypeToken<CollectionEntityResponse<SubscriberResponse>>() {
        }.getType();
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, collectionType);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

    /**
     * Wrapper method for pagination results
     *
     * @see com.apifon.sdk.model.PagingLinks
     * @see #getListByResouceUri(URI resourceUri)
     * @param paginationUrl the subscriber link to retrieve
     * @return
     * @throws ApifonRestException if an API exception occurs (Invalid request, Not found etc) 
     * @throws java.net.URISyntaxException if the URI provided is invalid.
     */
    public CollectionEntityResponse<SubscriberResponse> querySubscribers(String paginationUrl) throws ApifonRestException, URISyntaxException {
        return getListByResouceUri(new URI(paginationUrl));
    }
    
    /**
     * Subscribes the subscriber with the given lookupMD5
     * @param listId the list ID
     * @param lookupMD5 This will set the status to SUBSCRIBED according to the origin of the lookupMD5, <br>
     * for example if the lookupMD5 provided matches emailId on Subscriber this will set the email_status.
     * @return true if a subscriber was updated.
     * @throws ApifonRestException if an API exception occurs (Invalid request, Not found etc) 
     */
    public boolean subscribeSubscriber(String listId, String lookupMD5) throws ApifonRestException {
        this.setCurMethod("POST");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + lookupMD5 + "/subscribe", listId);
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        return this.isSuccessful(response);
    }
    
     /**
     * Unsubscribes the subscriber with the given lookupMD5
     * @param listId the list ID
     * @param lookupMD5 This will set the status to UNSUBSCRIBED according to the origin of the lookupMD5, <br>
     * for example if the lookupMD5 provided matches emailId on Subscriber this will set the email_status.
     * @return true if a subscriber was updated.
     * @throws ApifonRestException if an API exception occurs (Invalid request, Not found etc) 
     */
    public boolean unsubscribeSubscriber(String listId, String lookupMD5) throws ApifonRestException {
        this.setCurMethod("POST");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + lookupMD5 + "/unsubscribe", listId);
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        return this.isSuccessful(response);
    }

    /**
     * Removes the given subscriber from the list.<br><br>
     * <b>NOTE: This is a destructive irreversable operation</b>
     * @param listId the list ID
     * @param lookupMD5 a unique identifier for a subscriber.
     * @return true if a subscriber was removed.
     * @throws ApifonRestException if an API exception occurs (Invalid request, Not found etc) 
     */
    public boolean removeSubscriber(String listId, String lookupMD5) throws ApifonRestException {
        this.setCurMethod("DELETE");
        this.replacePlaceholderAndSetUrl(this.getEndpoint() + "/" + lookupMD5, listId);
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        return this.isSuccessful(response);
    }

    @Override
    public SubscriberResponse getByResourceUri(URI resourceUri) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.setCurEndpoint(resourceUri.toASCIIString());
        this.setCurBody("");
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, SubscriberResponse.class);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

    @Override
    public CollectionEntityResponse<SubscriberResponse> getListByResouceUri(URI resourceUri) throws ApifonRestException {
        Gson gson = new Gson();
        this.setCurMethod("GET");
        this.setCurEndpoint(resourceUri.toASCIIString());
        this.setCurBody("");
        Type collectionType = new TypeToken<CollectionEntityResponse<SubscriberResponse>>() {
        }.getType();
        HttpResponse response = this.dispatchToResponse();
        HttpEntity httpEntity = response.getEntity();
        try {
            String responseString = EntityUtils.toString(httpEntity);
            return gson.fromJson(responseString, collectionType);
        } catch (IOException ex) {
            Logger.getLogger(this.getClass().getName()).log(Level.SEVERE, "Exception while trying to parse response", ex);
            throw new ApifonRestException(ex);
        }
    }

}
