/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.resource.helpers;

import com.apifon.sdk.exception.OAuthTokenException;
import com.google.gson.annotations.SerializedName;
import java.util.Date;

/**
 *
 * @author Apifon
 */
public class OAuthCredentials implements Cloneable {

    @SerializedName("access_token")
    private String accessToken;
    @SerializedName("refresh_token")
    private String refreshToken;
    @SerializedName("access_code")
    private String accessCode;
    @SerializedName("expires_in")
    private Long expiresIn;
    private Date expirationDate;

    public static OAuthCredentials getOAuthCredentialsFromAccessCode(String accessCode) {
        OAuthCredentials credentials = new OAuthCredentials();
        credentials.setAccessCode(accessCode);
        return credentials;
    }

    public OAuthCredentials() {
    }

    public String getAccessToken() {
        return accessToken;
    }

    public void setAccessToken(String accessToken) {
        this.accessToken = accessToken;
    }

    public String getRefreshToken() {
        return refreshToken;
    }

    public void setRefreshToken(String refreshToken) {
        this.refreshToken = refreshToken;
    }

    public String getAccessCode() {
        return accessCode;
    }

    public void setAccessCode(String accessCode) {
        this.accessCode = accessCode;
    }

    public Long getExpiresIn() {
        return expiresIn;
    }

    public void setExpiresIn(Long expiresIn) throws OAuthTokenException {
        this.expiresIn = expiresIn;
        try {
            this.expirationDate = new Date(System.currentTimeMillis() + (expiresIn * 1000));
        } catch (NumberFormatException ex) {
            throw new OAuthTokenException("Failed to parsed expires in from value " + expiresIn);
        }
    }

    public Date getExpirationDate() {
        return new Date(expirationDate.getTime());
    }

    public void setExpirationDate(Date expirationDate) {
        this.expirationDate = expirationDate;
    }
    
    public boolean hasExpired() {
        if (this.expirationDate != null) {
            return new Date().after(this.expirationDate);
        }
        return true;
    }

    @Override
    public OAuthCredentials clone() throws CloneNotSupportedException {
        super.clone();
        OAuthCredentials copy = new OAuthCredentials();
        copy.accessCode = this.accessCode;
        copy.accessToken = this.accessToken;
        copy.expiresIn = this.expiresIn;
        copy.refreshToken = this.refreshToken;
        copy.expirationDate = this.expirationDate;
        return copy;
    }

    @Override
    public String toString() {
        return "OAuthCredentials{" + "accessToken=" + accessToken + ", refreshToken=" + refreshToken + ", accessCode=" + accessCode + ", expiresIn=" + expiresIn + ", expirationDate=" + expirationDate + '}';
    }

}
