/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package com.apifon.sdk.resource.helpers;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.Base64;

/**
 *
 * @author Apifon
 */
public class S256PKCEChallenge implements Cloneable {
    
    private final String codeVerifier;
    private final String codeChallenge;
    
    public S256PKCEChallenge(String codeVerifier) {
        try {
            byte[] codeVerifierBytes = codeVerifier.getBytes(StandardCharsets.US_ASCII);
            MessageDigest md = MessageDigest.getInstance("SHA-256");
            md.update(codeVerifierBytes, 0, codeVerifierBytes.length);
            this.codeVerifier = codeVerifier;
            this.codeChallenge = Base64.getUrlEncoder().encodeToString(md.digest()).split("=")[0];
        } catch (NoSuchAlgorithmException ex) {
            throw new IllegalStateException(ex);
        }
    }
    
    public static S256PKCEChallenge getRandomS256PKCEChallenge() {
        byte[] code = new byte[32];
        new SecureRandom().nextBytes(code);
        String randomCodeVerifier = Base64.getUrlEncoder().encodeToString(code).split("=")[0];
        return new S256PKCEChallenge(randomCodeVerifier);
    }

    public String getCodeVerifier() {
        return codeVerifier;
    }

    public String getCodeChallenge() {
        return codeChallenge;
    }

    @Override
    public S256PKCEChallenge clone() throws CloneNotSupportedException {
        super.clone();
        S256PKCEChallenge clone = new S256PKCEChallenge(this.codeVerifier);
        return clone;
    }
}
