/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package com.apifon.sdk.response;

import com.apifon.sdk.model.SingleResponse;
import com.apifon.sdk.model.request.CampaignDefaultsBuilder.CampaignDefaults;
import com.apifon.sdk.model.request.GdprPropertiesBuilder.GdprProperties;
import com.apifon.sdk.resource.helpers.FormCategory;
import com.apifon.sdk.resource.helpers.ListStatus;
import com.apifon.sdk.resource.helpers.ListType;
import com.apifon.sdk.serializer.DateAdapter;
import com.google.gson.annotations.JsonAdapter;
import com.google.gson.annotations.SerializedName;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 *
 * @author Apifon
 */
public class ListResponse extends SingleResponse {
    
    private String id;
    @SerializedName("account_id")
    private Long accountId;
    private ListType type;
    private String title;
    private ListStatus status;
    private String description;
    @JsonAdapter(DateAdapter.class)
    @SerializedName("date_created")
    private Date dateCreated;
    @JsonAdapter(DateAdapter.class)
    @SerializedName("date_updated")
    private Date dateUpdated;
    @SerializedName("campaign_defaults")
    private CampaignDefaults campaignDefaults;
    @SerializedName("total_subscribers")
    private Integer totalSubscribers;
    @SerializedName("active_subscribers")
    private Integer activeSubscribers;
    @SerializedName("active_email_subscribers")
    private Integer activeEmailSubscribers;
    @SerializedName("form_id")
    private String formId;
    @SerializedName("subscriber_fields")
    private List<SubscriberFieldResponse> subscriberFields;
    @SerializedName("gdpr_fields")
    private List<GdprFieldResponse> gdprFields;
    @SerializedName("brand_name")
    private String brandName;
    @SerializedName("company_name")
    private String companyName;
    @SerializedName("gdpr_properties")
    private GdprProperties gdprProperties;
    @SerializedName("form_urls")
    private Map<FormCategory, String> formUrls;
    private String prefix;
    private List<GroupResponse> groups;

    /**
     * Returns the subscriber fields of the list. May be null.<br>
     * <b>Note: The list is immutable</b>
     * @return
     */
    public List<SubscriberFieldResponse> getSubscriberFields() {
        return Collections.unmodifiableList(subscriberFields);
    }

    /**
     * Returns the gdpr fields of the list. May be null.<br>
     * <b>Note: The list is immutable</b>
     * @return
     */
    public List<GdprFieldResponse> getGdprFields() {
        return Collections.unmodifiableList(gdprFields);
    }

    /**
     * Returns the unique identifier of the list.
     * @return
     */
    public String getId() {
        return id;
    }

    /**
     * Returns the account ID of the list.
     * @return
     */
    public Long getAccountId() {
        return accountId;
    }

    /**
     * Returns the ListType of the list.
     * @see ListType
     * @return
     */
    public ListType getType() {
        return type;
    }

    /**
     * Returns the title of the list.
     * @return
     */
    public String getTitle() {
        return title;
    }

    /**
     * Returns the ListStatus of the list.
     * @return
     */
    public ListStatus getStatus() {
        return status;
    }

    /**
     * Returns the description of the list. May be null.
     * @return
     */
    public String getDescription() {
        return description;
    }

    /**
     * Returns the date created of the list.<br>
     * <b>Note: This Date is in GMT format</b>
     * @return
     */
    public Date getDateCreated() {
        return dateCreated;
    }

    /**
     * Returns the date updated of the list.<br>
     * <b>Note: This Date is in GMT format</b>
     * @return
     */
    public Date getDateUpdated() {
        return dateUpdated;
    }

    /**
     * Returns the CampaignDefaults of the list. May be null.
     * @see CampaignDefaults
     * @see com.apifon.sdk.model.request.CampaignDefaultsBuilder
     * @return
     */
    public CampaignDefaults getCampaignDefaults() {
        return campaignDefaults;
    }

    /**
     * Returns the total number of subscribers (Including UNSUBSCRIBED)
     * @return
     */
    public Integer getTotalSubscribers() {
        return totalSubscribers;
    }

    /**
     * Return the total number of subscribers that are SUBSCRIBED with a mobile destination.
     * @return
     */
    public Integer getActiveSubscribers() {
        return activeSubscribers;
    }

    /**
     * Return the total number of subscribers that are SUBSCRIBED with an email destination.
     * @return
     */
    public Integer getActiveEmailSubscribers() {
        return activeEmailSubscribers;
    }

    /**
     * Returns the unique form identifier of the list. May be null.
     * @return
     */
    public String getFormId() {
        return formId;
    }

    /**
     * Returns the brand name of the list.
     * @return
     */
    public String getBrandName() {
        return brandName;
    }

    /**
     * Returns the company name of the list.
     * @return
     */
    public String getCompanyName() {
        return companyName;
    }

    /**
     * Returns the GDPRProperties of the list. May be null.
     * @see GdprProperties
     * @see com.apifon.sdk.model.request.GdprPropertiesBuilder
     * @return
     */
    public GdprProperties getGdprProperties() {
        return gdprProperties;
    }

    /**
     * Returns a map containing the URLs of the list forms. May be null.
     * @return
     */
    public Map<FormCategory, String> getFormUrls() {
        return formUrls;
    }

    /**
     * Returns the default mobile prefix of the list
     * @return
     */
    public String getPrefix() {
        return prefix;
    }
    /**
     * Returns the groups of the list
     * @return
     */
    public List<GroupResponse> getGroups() {
        return groups;
    }
}
