/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.tests;

import com.apifon.sdk.Mookee;
import com.apifon.sdk.exception.OAuthException;
import com.apifon.sdk.model.IMChannel;
import com.apifon.sdk.model.ImRequest;
import com.apifon.sdk.model.MessageContent;
import com.apifon.sdk.model.SmsRequest;
import com.apifon.sdk.model.request.*;
import com.apifon.sdk.model.request.BounceRequestBuilder.BounceRequest;
import com.apifon.sdk.model.request.ListRequestBuilder.ListRequest;
import com.apifon.sdk.model.request.OAuthConfigurationBuilder.OAuthConfiguration;
import com.apifon.sdk.model.request.SubscriberRequestBuilder.SubscriberRequest;
import com.apifon.sdk.model.request.GroupRequestBuilder.GroupRequest;
import com.apifon.sdk.model.request.SubgroupRequestBuilder.SubgroupRequest;
import com.apifon.sdk.resource.helpers.*;
import com.apifon.sdk.tests.model.HMACToken;
import com.apifon.sdk.tests.model.OAuthClientCredentials;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Properties;
import java.util.UUID;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sun.org.apache.bcel.internal.generic.Select;
import org.junit.Before;

/**
 *
 * @author Apifon
 */
public class BaseTest {

    protected final String BLOCK_DESTINATION = "blocked@sdk.com";
    protected final String LIST_NAME = "SDK_TEST";
    protected final String GROUP_NAME = "my_group";
    protected final String SUBGROUP_NAME = "my_subgroup";
    protected final String ADD_DESTINATION = "306982345678";
    protected final String REFERENCE_ID = "REF_ID";

    protected String listId;
    protected String testHost;
    protected String identityServerHost;
    protected HMACToken smsToken;
    protected HMACToken imToken;
    protected OAuthClientCredentials clientCredentials;

    @Before
    public void setup() {
        Path resourceDirectory = Paths.get("src", "test", "resources", "test.properties");
        File testProperties = resourceDirectory.toFile();
        try (InputStream input = new FileInputStream(testProperties)) {
            Properties prop = new Properties();
            prop.load(input);
            testHost = prop.getProperty("test.host");
            identityServerHost = prop.getProperty("test.ids.host");
            listId = prop.getProperty("list.id");
            smsToken = new HMACToken(prop.getProperty("sms.token"), prop.getProperty("sms.secret"));
            imToken = new HMACToken(prop.getProperty("im.token"), prop.getProperty("im.secret"));
            clientCredentials = new OAuthClientCredentials(prop.getProperty("client_credentials.client_id"), prop.getProperty("client_credentials.client_secret"));
        } catch (IOException ex) {
            Logger.getLogger(BaseTest.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    protected void initMookee() {
        Mookee.setStaging(true, testHost, identityServerHost);
        Mookee.addCredentials("invalid", "abc", "123");
        Mookee.addCredentials("sms", smsToken.getToken(), smsToken.getSecret());
        Mookee.addCredentials("im", imToken.getToken(), imToken.getSecret());
    }

    protected void setMookeeToHMAC() {
        Mookee.setMookeeToHMACTokenMode();
    }
    
    protected void setValidOAuth(OpenIDScope... scopes) throws OAuthException {
        OAuthConfigurationBuilder.OAuthConfiguration oauthConfig
                = OAuthConfigurationBuilder.getClientCredentialsBuilder(this.clientCredentials.getClientId(), this.clientCredentials.getClientSecret())
                        .withScopes(Arrays.asList(scopes))
                        .withState(UUID.randomUUID().toString())
                        .build();
        setMookeeToOAuth(oauthConfig);
    }

    protected void setMookeeToOAuth(OAuthConfiguration config) throws OAuthException {
        String identifier = UUID.randomUUID().toString();
        Mookee.addOAuthResource(identifier, config);
        Mookee.retrieveClientCredentialsToken(identifier);
        Mookee.setMookeeToOAuthMode(identifier, true);
    }

    protected BounceRequest getAddBounceRequest() {
        return BounceRequestBuilder
                .getBounceRequestBuilder(BLOCK_DESTINATION, BounceReason.SUBSCRIBER_REQUESTED)
                .withDescription("SDK_TEST")
                .build();
    }

    protected ListRequest getCreateListRequest() {
        return ListRequestBuilder
                .getAddListRequestBuilder(ListType.ADVANCED, LIST_NAME, "30")
                .withDescription("SDK_TEST")
                .withTitle(LIST_NAME)
                .withCampaignDefaults(CampaignDefaultsBuilder.getCampaignDefaultsBuilder())
                .build();
    }

    protected SubscriberRequest getCreateSubscriberRequest() {
        return SubscriberRequestBuilder
                .getAddSubscriberRequestBuilder(listId)
                .withDestination(ADD_DESTINATION, SubscriberStatus.SUBSCRIBED)
                .withSubscriberField("FIRST_NAME", UUID.randomUUID().toString())
                .withSubscriberField("LAST_NAME", UUID.randomUUID().toString())
                .build();
    }

    protected GroupRequest getCreateGroupRequest(){
        return GroupRequestBuilder.getAddGroupRequestBuilder(listId, GroupType.GROUP,GROUP_NAME)
                .withGroupOptionsOrientation(GroupOptionsOrientation.HORIZONTAL)
                .withInternal(false)
                .withSelectionType(SelectionType.MULTIPLE)
                .build();
    }

    protected  SubgroupRequest getCreateSubgroupRequest(String groupField){
        return SubgroupRequestBuilder.getAddSubgroupsRequestBuilder(listId,groupField,SUBGROUP_NAME).build();
    }


    protected SmsRequest getSendSMSRequest() {
        SmsRequest smsRequest = new SmsRequest();
        MessageContent content = new MessageContent();
        content.setDc(0);
        content.setSenderId("SDK_Test");
        content.setText("AUTOMATED_MESSAGE");
        smsRequest.addStrSubscriber(ADD_DESTINATION);
        smsRequest.setMessage(content);
        smsRequest.setReferenceId(REFERENCE_ID);
        return smsRequest;
    }

    protected ImRequest getSendIMRequest() {
        ImRequest imRequest = new ImRequest();
        IMChannel channel = new IMChannel();
        channel.setSenderId("Mookee");
        channel.setText("AUTOMATED_MESSAGE");
        channel.setTtl(3600);
        imRequest.addStrSubscriber(ADD_DESTINATION);
        imRequest.setReferenceId(REFERENCE_ID);
        imRequest.setImChannels(Arrays.asList(channel));
        return imRequest;
    }

}
