/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.apifon.sdk.tests;

import com.apifon.sdk.Mookee;
import com.apifon.sdk.exception.ApifonRestException;
import com.apifon.sdk.model.request.GroupRequestBuilder;
import com.apifon.sdk.resource.*;
import com.apifon.sdk.response.*;

import java.net.URI;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

/**
 *
 * @author Apifon
 */
public class HmacAPITest extends BaseTest {

    @Before
    public void init() {
        super.setup();
        initMookee();
        setMookeeToHMAC();
    }

    @Test(expected = ApifonRestException.class)
    public void test_invalidToken_API() throws ApifonRestException {
        Mookee.setActiveCredential("invalid");
        BounceListResource resource = new BounceListResource();
        try {
            resource.addBounceSubscriber(getAddBounceRequest());
        } catch (ApifonRestException ex) {
            Assert.assertEquals(401, ex.getResp().status);
            Assert.assertEquals("Unauthorized", ex.getResp().title);
            throw ex;
        }
    }

    @Test(expected = Test.None.class)
    public void test_BalanceAPI() throws ApifonRestException {
        Mookee.setActiveCredential("sms");
        BalanceResource resource = new BalanceResource();
        BalanceResponse balanceResponse = resource.send();
        Assert.assertNotNull(balanceResponse.getBalance());
        Assert.assertFalse(Float.valueOf(balanceResponse.getBalance()) < 0);
        Assert.assertEquals(0.0F, Float.valueOf(balanceResponse.getPlafon()), 0.0F);
        Assert.assertEquals(1.0F, Float.valueOf(balanceResponse.getReserved()), 0.0F);
        Assert.assertTrue(balanceResponse.getSubscriptions().isEmpty());
    }

    @Test(expected = Test.None.class)
    public void test_Add_thenRetrieve_thenDelete_BounceAPI() throws ApifonRestException {
        Mookee.setActiveCredential("im");
        BounceListResource resource = new BounceListResource();
        // Add
        URI addBounceSubscriber = resource.addBounceSubscriber(getAddBounceRequest());
        Assert.assertNotNull(addBounceSubscriber);
        // Retrieve
        BounceResponse addedBounce = resource.getByResourceUri(addBounceSubscriber);
        Assert.assertEquals(BLOCK_DESTINATION, addedBounce.getDestination());
        // Delete
        boolean removed = resource.removeBounceSubscriber(addedBounce.getDestination());
        Assert.assertTrue(removed);
    }

    @Test(expected = Test.None.class)
    public void test_Add_thenRetrieve_thenDelete_ListAPI() throws ApifonRestException {
        Mookee.setActiveCredential("sms");

        ListResource resource = new ListResource();
        // Add
        URI createdListURI = resource.createList(getCreateListRequest());
        Assert.assertNotNull(createdListURI);
        // Retrieve
        ListResponse list = resource.getByResourceUri(createdListURI);
        Assert.assertNotNull(list);
        Assert.assertEquals(LIST_NAME, list.getTitle());
        // Delete
        boolean removed = resource.removeList(list.getId());
        Assert.assertTrue(removed);
    }

    @Test(expected = Test.None.class)
    public void test_Add_thenRetrieve_thenDelete_SubscriberAPI() throws ApifonRestException {
        Mookee.setActiveCredential("sms");
        SubscriberResource resource = new SubscriberResource();
        // Add
        URI createdSubscriberURI = resource.addSubscriber(getCreateSubscriberRequest());
        Assert.assertNotNull(createdSubscriberURI);
        // Retrieve
        SubscriberResponse subscriber = resource.getByResourceUri(createdSubscriberURI);
        Assert.assertNotNull(subscriber);
        Assert.assertEquals(ADD_DESTINATION, subscriber.getDestination());
        Assert.assertNotNull(subscriber.getSubscriberFields().get("FIRST_NAME"));
        Assert.assertNotNull(subscriber.getSubscriberFields().get("LAST_NAME"));
        // Delete
        boolean removed = resource.removeSubscriber(subscriber.getListId(), subscriber.getDestinationId());
        Assert.assertTrue(removed);
    }

    @Test(expected = Test.None.class)
    public void test_Add_thenRetrieve_thenDelete_GroupsAPI_andSubgroupsApi() throws ApifonRestException {
        Mookee.setActiveCredential("sms");

        GroupResource groupResource = new GroupResource();
        SubgroupResource subgroupResource = new SubgroupResource();
        //Add Group
        URI createdGroupURI = groupResource.addGroup(getCreateGroupRequest());
        Assert.assertNotNull(createdGroupURI);
        //Retrieve Group
        GroupResponse group = groupResource.getByResourceUri(createdGroupURI);
        Assert.assertNotNull(group);
        Assert.assertEquals(GROUP_NAME,group.getName());
        //Add subgroup
        URI createdSubgroupURI = subgroupResource.addSubgroup(getCreateSubgroupRequest(group.getField()));
        Assert.assertNotNull(createdSubgroupURI);
        //delete group
        boolean removed = groupResource.removeGroup(listId, group.getField());
        Assert.assertTrue(removed);



    }

    @Test(expected = Test.None.class)
    public void test_smsGateway() throws ApifonRestException {
        Mookee.setActiveCredential("sms");
        SMSResource resource = new SMSResource();
        GatewayResponse send = resource.send(getSendSMSRequest());
        Assert.assertEquals(REFERENCE_ID, send.getReferenceId());
        Assert.assertEquals(200, send.getResultInfo().getStatusCode());
    }

    @Test(expected = Test.None.class)
    public void test_imGateway() throws ApifonRestException {
        Mookee.setActiveCredential("im");
        IMResource resource = new IMResource();
        GatewayResponse send = resource.send(getSendIMRequest());
        Assert.assertEquals(REFERENCE_ID, send.getReferenceId());
        Assert.assertEquals(200, send.getResultInfo().getStatusCode());
    }

}
